@@ -3,7 +3,7 @@
 /*
     Rosegarden
     A sequencer and musical notation editor.
-    Copyright 2000-2018 the Rosegarden development team.
+    Copyright 2000-2024 the Rosegarden development team.
     See the AUTHORS file for more details.
 
     This program is free software; you can redistribute it and/or
@@ -28,14 +28,13 @@
  *
  * This file contains definitions of several classes to assist in
  * creating and manipulating certain event types.  The classes are:
- * 
+ *
  *   Accidental
  *   Clef
  *   Key
  *   Indication
  *   Pitch
  *   Note
- *   TimeSignature
  *   AccidentalTable
  *   Symbol
  *   GeneratedRegion
@@ -54,12 +53,12 @@
  * as the Note methods that calculate duration-related values without
  * reference to any specific pitch or other note-event properties; or
  * everything in Pitch).
- * 
+ *
  * This file also defines the event types and standard property names
  * for the basic events.
  */
 
-namespace Rosegarden 
+namespace Rosegarden
 {
 
 extern ROSEGARDENPRIVATE_EXPORT const int MIN_SUBORDERING;
@@ -92,9 +91,9 @@ namespace Accidentals
 
     typedef std::vector<Accidental> AccidentalList;
 
-	/** 
+	/**
 	 * When no accidental is specified for a pitch, there are several
-	 * strategies to determine what accidental to display for an 
+	 * strategies to determine what accidental to display for an
 	 * out-of-key pitch
 	 */
 	enum NoAccidentalStrategy {
@@ -103,7 +102,7 @@ namespace Accidentals
 		/** always use flats */
 		UseFlats,
 		/** always use sharps or always use flats depending on of what
-		 * type of accidentals the current key is made up */ 
+		 * type of accidentals the current key is made up */
 		UseKeySharpness,
 		/** use the most likely accidental for this key */
 		UseKey
@@ -113,7 +112,7 @@ namespace Accidentals
      * Get the predefined accidentals (i.e. the ones listed above)
      * in their defined order.
      */
-    extern ROSEGARDENPRIVATE_EXPORT AccidentalList getStandardAccidentals();
+     // unused extern ROSEGARDENPRIVATE_EXPORT AccidentalList getStandardAccidentals();
 
     /**
      * Get the change in pitch resulting from an accidental: -1 for
@@ -129,7 +128,7 @@ namespace Accidentals
      * Get the Accidental corresponding to a change in pitch: flat
      * for -1, double-sharp for 2, natural for 0 etc.
      *
-     * Useful for tying to code that represents accidentals by 
+     * Useful for tying to code that represents accidentals by
      * their pitch change.
      */
     extern ROSEGARDENPRIVATE_EXPORT Accidental getAccidental(int pitchChange);
@@ -141,8 +140,8 @@ namespace Accidentals
  */
 
 typedef std::string Mark;
-  
-namespace Marks //!!! This would be better as a class, these days
+
+namespace Marks
 {
     extern ROSEGARDENPRIVATE_EXPORT const Mark NoMark;         // " "
 
@@ -216,7 +215,7 @@ namespace Marks //!!! This would be better as a class, these days
     /**
      * Extract the number of marks from an event.
      */
-    extern ROSEGARDENPRIVATE_EXPORT int getMarkCount(const Event &e);
+    // unused extern ROSEGARDENPRIVATE_EXPORT int getMarkCount(const Event &e);
 
     /**
      * Extract the marks from an event.
@@ -295,7 +294,7 @@ public:
      * returns the default clef (with a warning).  You should normally
      * test Clef::isValid() to catch that before construction.
      */
-    Clef(const Event &e);
+    explicit Clef(const Event &e);
 
     /**
      * Construct a Clef from the given data.  Throws a BadClefName
@@ -335,7 +334,7 @@ public:
      */
     int getOctaveOffset() const { return m_octaveOffset; }
 
-    /** 
+    /**
      * Return the number of semitones a pitch in the treble clef would
      * have to be lowered by in order to be drawn with the same height
      * and accidental in this clef
@@ -406,13 +405,13 @@ public:
      * returns the default key (with a warning).  You should normally
      * test Key::isValid() to catch that before construction.
      */
-    Key(const Event &e);
+    explicit Key(const Event &e);
 
     /**
      * Construct the named key.  Throws a BadKeyName exception if the
      * given string does not match one of the known key names.
      */
-    Key(const std::string &name);
+    explicit Key(const std::string &name);
 
     /**
      * Construct a key from signature and mode.  May throw a
@@ -478,7 +477,7 @@ public:
     int getTonicPitch() const {
         return m_keyDetailMap[m_name].m_tonicPitch;
     }
-    
+
     /**
      * Return the number of sharps or flats in the key's signature.
      */
@@ -488,7 +487,7 @@ public:
 
     /**
      * Return the key with the same signature but different
-     * major/minor mode.  For example if called on C major, 
+     * major/minor mode.  For example if called on C major,
      * returns A minor.
      */
     Key getEquivalent() const {
@@ -536,7 +535,7 @@ public:
      * same status in terms of accidentals as it had when
      * found in the given previous key.
      */
-    int convertFrom(int pitch, const Key &previousKey,
+    int convertFrom(int p, const Key &previousKey,
                     const Accidental &explicitAccidental =
                     Accidentals::NoAccidental) const;
 
@@ -571,7 +570,7 @@ public:
 
     /**
      * Transpose this key by the specified interval given in pitch and steps
-     * 
+     *
      * For example: transposing F major by a major triad (4,2) yields
      *  A major.
      */
@@ -592,8 +591,8 @@ private:
         KeyDetails(); // ctor needed in order to live in a map
 
         KeyDetails(bool sharps, bool minor, int sharpCount,
-                   std::string equivalence, std::string rg2name,
-                                   int m_tonicPitch);
+                   const std::string& equivalence, const std::string& rg2name,
+                   int tonicPitch);
 
         KeyDetails(const KeyDetails &d);
 
@@ -640,7 +639,7 @@ public:
     static const std::string FigParameterChord;
     static const std::string Figuration;
 
-    Indication(const Event &e)
+    explicit Indication(const Event &e)
         /* throw (Event::NoData, Event::BadType) */;
     Indication(const std::string &s, timeT indicationDuration)
         /* throw (BadIndicationName) */;
@@ -674,7 +673,7 @@ public:
     Event *getAsEvent(timeT absoluteTime) const;
 
 private:
-    bool isValid(const std::string &s) const;
+    static bool isValid(const std::string &s);
 
     std::string m_indicationType;
     timeT m_duration;
@@ -727,8 +726,8 @@ public:
     static const std::string Tiny;        // begin \tiny font section
     static const std::string Small;       // begin \small font section
     static const std::string NormalSize;  // begin \normalsize font section
-    
-    Text(const Event &e)
+
+    explicit Text(const Event &e)
         /* throw (Event::NoData, Event::BadType) */;
     Text(const std::string &text,
          const std::string &textType = UnspecifiedType);
@@ -740,10 +739,10 @@ public:
     std::string getTextType() const { return m_type; }
 
     // Relevant for lyrics, and borrowed for figuration IDs.
-    int getVerse() const { return m_verse; } 
+    int getVerse() const { return m_verse; }
     void setVerse(int verse) { m_verse = verse; }
 
-    static bool isTextOfType(Event *, std::string type);
+    static bool isTextOfType(Event *, const std::string& type);
 
     /**
      * Return those text types that the user should be allowed to
@@ -774,7 +773,7 @@ private:
  *
  * (See docs/discussion/units.txt for explanation of pitch units.)
  *
- * This completely replaces the older NotationDisplayPitch class. 
+ * This completely replaces the older NotationDisplayPitch class.
  */
 
 class ROSEGARDENPRIVATE_EXPORT Pitch
@@ -786,13 +785,13 @@ public:
      * absent, NoData is thrown.  The BaseProperties::ACCIDENTAL
      * property will also be used if present.
      */
-    Pitch(const Event &e)
+    explicit Pitch(const Event &e)
         /* throw Event::NoData */;
 
     /**
      * Construct a Pitch object based on the given performance (MIDI) pitch.
      */
-    Pitch(int performancePitch, 
+    Pitch(int performancePitch,
           const Accidental &explicitAccidental = Accidentals::NoAccidental);
 
     /**
@@ -823,7 +822,7 @@ public:
           int octaveBase = -2);
 
     /**
-     * Construct a Pitch based on (MIDI) octave, note in the C major scale and 
+     * Construct a Pitch based on (MIDI) octave, note in the C major scale and
      * performance pitch. The accidental is calculated based on these
      * properties.
      */
@@ -840,7 +839,7 @@ public:
     Pitch(char noteName, int octave, const Key &key,
           const Accidental &explicitAccidental = Accidentals::NoAccidental,
           int octaveBase = -2);
-    
+
     /**
      * Construct a Pitch corresponding a staff line or space on a
      * classical 5-line staff.  The bottom staff line has height 0,
@@ -937,11 +936,11 @@ public:
     int getOctave(int octaveBase = -2) const;
 
     /**
-     * Return the octave containing this pitch, including the accidentals. 
+     * Return the octave containing this pitch, including the accidentals.
      * The octaveBase argument specifies the octave containing MIDI pitch 0;
      * middle-C is in octave octaveBase + 5.
      */
-    int getOctaveAccidental(int octaveBase = -2, Accidental acc = Accidentals::NoAccidental) const;
+    int getOctaveAccidental(int octaveBase = -2, const Accidental& acc = Accidentals::NoAccidental) const;
 
     /**
      * Return the pitch within the octave, in the range 0 to 11.
@@ -952,7 +951,7 @@ public:
      * Return whether this pitch is diatonic in the given key.
      */
     bool isDiatonicInKey(const Key &key) const;
-    
+
     /**
      * Return a reference name for this pitch.  (C 4, Bb 2, etc...)
      * using the key of C major explicitly, which should allow the accidentals
@@ -996,13 +995,13 @@ public:
      * specified interval in the given key. The key is left unchanged,
      * only the pitch is transposed.
      */
-    Pitch transpose(const Key &key, int pitchDelta, int heightDelta);
+    Pitch transpose(const Key &key, int pitchDelta, int heightDelta) const;
 
-    /** 
+    /**
       * checks whether the accidental specified for this pitch (if any)
       * is valid - for example, a Sharp for pitch 11 is invalid, as
       * it's between A# and B#.
-      */  
+      */
     bool validAccidental() const;
 
     /**
@@ -1010,12 +1009,22 @@ public:
      */
     Event *getAsNoteEvent(timeT absoluteTime, timeT duration) const;
 
+    /**
+     * Get the major key that has this Pitch as the tonic
+     */
+    Key getAsKey() const;
+
+    /**
+     * Get the major or minor key that has this Pitch as the tonic
+     */
+    Key getAsKey(bool isMinor) const;
+
 private:
     int m_pitch;
     Accidental m_accidental;
 
     static void rawPitchToDisplayPitch
-    (int, const Clef &, const Key &, int &, Accidental &, 
+    (int, const Clef &, const Key &, int &, Accidental &,
     Accidentals::NoAccidentalStrategy);
 
     static void displayPitchToRawPitch
@@ -1025,9 +1034,6 @@ private:
 
 
 
-class TimeSignature;
-
-
 /**
  * The Note class represents note durations only, not pitch or
  * accidental; it's therefore just as relevant to rest events as to
@@ -1107,9 +1113,9 @@ public:
     Event *getAsNoteEvent(timeT absoluteTime, int pitch) const;
 
     /// Returned event is on heap; caller takes responsibility for ownership
-    Event *getAsRestEvent(timeT absoluteTime) const;
+    // unused Event *getAsRestEvent(timeT absoluteTime) const;
+
 
-  
 private:
     Type m_type;
     int m_dots;
@@ -1123,185 +1129,6 @@ private:
 
 
 
-/**
- * TimeSignature contains arithmetic methods relevant to time
- * signatures and bar durations, including code for splitting long
- * rest intervals into bite-sized chunks.  Although there is a time
- * signature Event type, these Events don't appear in regular Segments
- * but only in the Composition's reference segment.
- */
-
-class ROSEGARDENPRIVATE_EXPORT TimeSignature
-{
-public:
-    static const TimeSignature DefaultTimeSignature;
-    typedef Exception BadTimeSignature;
-
-    TimeSignature() :
-        m_numerator(DefaultTimeSignature.m_numerator),
-        m_denominator(DefaultTimeSignature.m_denominator),
-        m_common(false), m_hidden(false), m_hiddenBars(false) { }
-
-    /**
-     * Construct a TimeSignature object describing a time signature
-     * with the given numerator and denominator.  If preferCommon is
-     * true and the time signature is a common or cut-common time, the
-     * constructed object will return true for isCommon; if hidden is
-     * true, the time signature is intended not to be displayed and
-     * isHidden will return true; if hiddenBars is true, the bar lines
-     * between this time signature and the next will not be shown.
-     */
-    TimeSignature(int numerator, int denominator,
-                  bool preferCommon = false,
-                  bool hidden = false,
-                  bool hiddenBars = false)
-        /* throw (BadTimeSignature) */;
-    
-    TimeSignature(const TimeSignature &ts) :
-        m_numerator(ts.m_numerator),
-        m_denominator(ts.m_denominator),
-        m_common(ts.m_common),
-        m_hidden(ts.m_hidden),
-        m_hiddenBars(ts.m_hiddenBars) { }
-
-    ~TimeSignature() { }
-
-    TimeSignature &operator=(const TimeSignature &ts);
-
-    bool operator==(const TimeSignature &ts) const {
-        return ts.m_numerator == m_numerator && ts.m_denominator == m_denominator;
-    }
-    bool operator!=(const TimeSignature &ts) const {
-        return !operator==(ts);
-    }
-    bool operator<(const TimeSignature &ts) const {
-        // We don't really need to ordered time signatures, but to be able to
-        // create a map keyed with time signatures. We want to distinguish
-        // 4/4 from 2/4 as well as 4/4 from 2/2.
-        double ratio1 = (double) m_numerator / (double) m_denominator;
-        double ratio2 = (double) ts.m_numerator / (double) ts.m_denominator;
-        if (ratio1 == ratio2) return m_denominator > ts.m_denominator;
-        else return ratio1 < ratio2;
-    }
-
-    int getNumerator()     const { return m_numerator; }
-    int getDenominator()   const { return m_denominator; }
-
-    bool isCommon()        const { return m_common; }
-    bool isHidden()        const { return m_hidden; }
-    bool hasHiddenBars()   const { return m_hiddenBars; }
-
-    void setHidden(bool hidden) { m_hidden = hidden; }
-
-    timeT getBarDuration() const;
-
-    /**
-     * Return the unit of the time signature.  This is the note
-     * implied by the denominator.  For example, the unit of 4/4 time
-     * is the crotchet, and that of 6/8 is the quaver.  (The numerator
-     * of the time signature gives the number of units per bar.)
-     */
-    Note::Type getUnit()  const;
-
-    /**
-     * Return the duration of the unit of the time signature.
-     * See also getUnit().  In most cases getBeatDuration() gives
-     * a more meaningful value.
-     */
-    timeT getUnitDuration() const;
-
-    /**
-     * Return true if this time signature indicates dotted time.
-     */
-    bool isDotted() const;
-
-    /**
-     * Return the duration of the beat of the time signature.  For
-     * example, the beat of 4/4 time is the crotchet, the same as its
-     * unit, but that of 6/8 is the dotted crotchet (there are only
-     * two beats in a 6/8 bar).  The beat therefore depends on whether
-     * the signature indicates dotted or undotted time.
-     */
-    timeT getBeatDuration() const;
-
-    /**
-     * Return the number of beats in a complete bar.
-     */
-    int getBeatsPerBar()  const {
-        return getBarDuration() / getBeatDuration();
-    }
-
-    /**
-     * Get the "optimal" list of rest durations to make up a bar in
-     * this time signature.
-     */
-    void getDurationListForBar(DurationList &dlist) const;
-
-    /**
-     * Get the "optimal" list of rest durations to make up a time
-     * interval of the given total duration, starting at the given
-     * offset after the start of a bar, assuming that the interval
-     * is entirely in this time signature.
-     */
-    void getDurationListForInterval(DurationList &dlist,
-                                    timeT intervalDuration,
-                                    timeT startOffset = 0) const;
-
-    /**
-     * Get the level of emphasis for a position in a bar. 4 is lots
-     * of emphasis, 0 is none.
-     */
-    int getEmphasisForTime(timeT offset);
-
-    /**
-     * Return a list of divisions, subdivisions, subsubdivisions
-     * etc of a bar in this time, up to the given depth.  For example,
-     * if the time signature is 6/8 and the depth is 3, return a list
-     * containing 2, 3, and 2 (there are 2 beats to the bar, each of
-     * which is best subdivided into 3 subdivisions, each of which
-     * divides most neatly into 2).
-     */
-    void getDivisions(int depth, std::vector<int> &divisions) const;
-
-private:
-    friend class Composition;
-    friend class TimeTempoSelection;
-
-    TimeSignature(const Event &e)
-        /* throw (Event::NoData, Event::BadType, BadTimeSignature) */;
-
-    static const std::string EventType;
-    static const int EventSubOrdering;
-    static const PropertyName NumeratorPropertyName;
-    static const PropertyName DenominatorPropertyName;
-    static const PropertyName ShowAsCommonTimePropertyName;
-    static const PropertyName IsHiddenPropertyName;
-    static const PropertyName HasHiddenBarsPropertyName;
-
-    /// Returned event is on heap; caller takes responsibility for ownership
-    Event *getAsEvent(timeT absoluteTime) const;
-
-private:
-    int m_numerator;
-    int m_denominator;
-
-    bool m_common;
-    bool m_hidden;
-    bool m_hiddenBars;
-
-    mutable int  m_barDuration;
-    mutable int  m_beatDuration;
-    mutable int  m_beatDivisionDuration;
-    mutable bool m_dotted;
-    void setInternalDurations() const;
-
-    // a time & effort saving device
-    static const timeT m_crotchetTime;
-    static const timeT m_dottedCrotchetTime;
-};
-
-
-
 /**
  * AccidentalTable represents a set of accidentals in force at a
  * given time.
@@ -1334,9 +1161,9 @@ public:
         BarResetExplicit    // c# | c -> add natural to c
     };
 
-    AccidentalTable(const Key & = Key(), const Clef & = Clef(),
-                    OctaveType = OctavesCautionary,
-                    BarResetType = BarResetCautionary);
+    explicit AccidentalTable(const Key & = Key(), const Clef & = Clef(),
+                             OctaveType = OctavesCautionary,
+                             BarResetType = BarResetCautionary);
 
     AccidentalTable(const AccidentalTable &);
     AccidentalTable &operator=(const AccidentalTable &);
@@ -1349,7 +1176,7 @@ public:
 
     void newBar();
     void newClef(const Clef &);
-    
+
 private:
     Key m_key;
     Clef m_clef;
@@ -1358,7 +1185,7 @@ private:
 
     struct AccidentalRec {
         AccidentalRec() : accidental(Accidentals::NoAccidental), previousBar(false) { }
-        AccidentalRec(Accidental a, bool p) : accidental(a), previousBar(p) { }
+        AccidentalRec(const Accidental& a, bool p) : accidental(a), previousBar(p) { }
         Accidental accidental;
         bool previousBar;
     };
@@ -1372,7 +1199,7 @@ private:
     AccidentalMap m_newCanonicalAccidentals;
 };
 
- 
+
 /** Definitions for use in the Symbol event type
  *
  * A Symbol has no duration, and the things it represents will probably always
@@ -1395,16 +1222,16 @@ public:
     static const std::string Coda;
     static const std::string Breath;
 
-    Symbol(const Event &e)
+    explicit Symbol(const Event &e)
         /* throw (Event::NoData, Event::BadType) */;
-    Symbol(const std::string &symbolType = UnspecifiedType);
+    explicit Symbol(const std::string &symbolType = UnspecifiedType);
     Symbol(const Symbol &);
     Symbol &operator=(const Symbol &);
     ~Symbol ();
 
     std::string getSymbolType() const { return m_type; }
 
-    static bool isSymbolOfType(Event *, std::string type);
+    // unused static bool isSymbolOfType(Event *, const std::string& type);
 
     /// Returned event is on heap; caller takes responsibility for ownership
     Event *getAsEvent(timeT absoluteTime) const;
