@@ -3,7 +3,7 @@
 /*
     Rosegarden
     A sequencer and musical notation editor.
-    Copyright 2000-2018 the Rosegarden development team.
+    Copyright 2000-2024 the Rosegarden development team.
     See the AUTHORS file for more details.
 
     This program is free software; you can redistribute it and/or
@@ -14,6 +14,7 @@
 */
 
 #define RG_MODULE_STRING "[Studio]"
+#define RG_NO_DEBUG_PRINT
 
 #include <iostream>
 
@@ -23,6 +24,7 @@
 #include "SoftSynthDevice.h"
 #include "Instrument.h"
 
+#include "base/RecordIn.h"
 #include "base/Segment.h"
 #include "misc/Strings.h"
 #include "Track.h"
@@ -46,6 +48,7 @@ Studio::Studio() :
     amwShowSynthFaders(true),
     amwShowAudioSubmasters(true),
     amwShowUnassignedFaders(false),
+    m_audioInputs(0),
     m_midiThruFilter(0),
     m_midiRecordFilter(0),
     m_metronomeDevice(0)
@@ -129,15 +132,31 @@ Studio::removeDevice(DeviceId id)
 }
 
 void
-Studio::
-resyncDeviceConnections()
+Studio::resyncDeviceConnections()
 {
-    // Sync all the device connections
+    // Sync all the MidiDevice connections to the current connections
+    // according to RosegardenSequencer.
+
     DeviceList *devices = getDevices();
-    for (uint i = 0; i < devices->size(); ++i) {
-        DeviceId id = (*devices)[i]->getId();
-        QString connection = RosegardenSequencer::getInstance()->getConnection(id);
-        (*devices)[i]->setConnection(qstrtostr(connection));
+
+    // For each Device
+    for (unsigned i = 0; i < devices->size(); ++i) {
+        // Only MidiDevice's have connections.
+        MidiDevice *midiDevice = dynamic_cast<MidiDevice *>((*devices)[i]);
+        if (!midiDevice)
+            continue;
+
+        DeviceId deviceId = midiDevice->getId();
+        QString connection =
+                RosegardenSequencer::getInstance()->getConnection(deviceId);
+
+        // If we are connected to something, but the user didn't ask for
+        // anything, we must have been connected up by
+        // AlsaDriver::connectSomething().  In that case, we'd better store
+        // the connection as the user selection in case the user actually
+        // likes it.
+        if (connection != ""  &&  midiDevice->getUserConnection() == "")
+            midiDevice->setUserConnection(qstrtostr(connection));
     }
 }
 
@@ -223,20 +242,22 @@ Studio::getPresentationInstruments() const
     return list;
 }
 
-Instrument*
-Studio::getInstrumentById(InstrumentId id)
+Instrument *
+Studio::getInstrumentById(InstrumentId id) const
 {
-    std::vector<Device*>::iterator it;
-    InstrumentList list;
-    InstrumentList::iterator iit;
-
-    for (it = m_devices.begin(); it != m_devices.end(); ++it)
+    // For each Device
+    for (std::vector<Device *>::const_iterator deviceIter = m_devices.begin();
+         deviceIter != m_devices.end();
+         ++deviceIter)
     {
-        list = (*it)->getAllInstruments();
+        InstrumentList list = (*deviceIter)->getAllInstruments();
 
-        for (iit = list.begin(); iit != list.end(); ++iit)
-            if ((*iit)->getId() == id)
-                return (*iit);
+        for (InstrumentList::const_iterator instrumentIter = list.begin();
+             instrumentIter != list.end();
+             ++instrumentIter) {
+            if ((*instrumentIter)->getId() == id)
+                return (*instrumentIter);
+        }
     }
 
     return nullptr;
@@ -281,7 +302,7 @@ Studio::getInstrumentFromList(int index)
 }
 
 Instrument *
-Studio::getInstrumentFor(Segment *segment)
+Studio::getInstrumentFor(const Segment *segment) const
 {
     if (!segment) return nullptr;
     if (!segment->getComposition()) return nullptr;
@@ -292,7 +313,7 @@ Studio::getInstrumentFor(Segment *segment)
 }
 
 Instrument *
-Studio::getInstrumentFor(Track *track)
+Studio::getInstrumentFor(const Track *track) const
 {
     if (!track) return nullptr;
     InstrumentId iid = track->getInstrument();
@@ -438,21 +459,6 @@ Studio::setRecordInCount(unsigned newRecordInCount)
     // The mapped IDs get set by RosegardenDocument::initialiseStudio().
 }
 
-// Clear down the devices  - the devices will clear down their
-// own Instruments.
-//
-void
-Studio::clear()
-{
-    InstrumentList list;
-    std::vector<Device*>::iterator it;
-
-    for (it = m_devices.begin(); it != m_devices.end(); ++it)
-        delete *it;
-
-    m_devices.erase(m_devices.begin(), m_devices.end());
-}
-
 std::string
 Studio::toXmlString() const
 {
@@ -511,31 +517,35 @@ Studio::toXmlString(const std::vector<DeviceId> &devices) const
     return studio.str();
 }
 
-// Run through the Devices checking for MidiDevices and
-// returning the first Metronome we come across
-//
-const MidiMetronome*
+const MidiMetronome *
 Studio::getMetronomeFromDevice(DeviceId id)
 {
-    std::vector<Device*>::iterator it;
+    // For each Device
+    for (std::vector<Device *>::const_iterator deviceIter = m_devices.begin();
+         deviceIter != m_devices.end();
+         ++deviceIter) {
 
-    for (it = m_devices.begin(); it != m_devices.end(); ++it) {
+        //RG_DEBUG << "getMetronomeFromDevice(): Having a look at device " << (*it)->getId();
 
-        RG_DEBUG << "getMetronomeFromDevice(): Having a look at device " << (*it)->getId();
+        // No ID match?  Try the next.
+        if ((*deviceIter)->getId() != id)
+            continue;
 
-        MidiDevice *midiDevice = dynamic_cast<MidiDevice*>(*it);
-        if (midiDevice && 
-            midiDevice->getId() == id &&
+        MidiDevice *midiDevice = dynamic_cast<MidiDevice *>(*deviceIter);
+
+        // If it's a MidiDevice and it has a metronome, return it.
+        if (midiDevice  &&
             midiDevice->getMetronome()) {
-            RG_DEBUG << "getMetronomeFromDevice(" << id << "): device is a MIDI device";
+            //RG_DEBUG << "getMetronomeFromDevice(" << id << "): device is a MIDI device";
             return midiDevice->getMetronome();
         }
 
-        SoftSynthDevice *ssDevice = dynamic_cast<SoftSynthDevice *>(*it);
-        if (ssDevice && 
-            ssDevice->getId() == id &&
+        SoftSynthDevice *ssDevice = dynamic_cast<SoftSynthDevice *>(*deviceIter);
+
+        // If it's a SoftSynthDevice and it has a metronome, return it.
+        if (ssDevice  &&
             ssDevice->getMetronome()) {
-            RG_DEBUG << "getMetronomeFromDevice(" << id << "): device is a soft synth device";
+            //RG_DEBUG << "getMetronomeFromDevice(" << id << "): device is a soft synth device";
             return ssDevice->getMetronome();
         }
     }
@@ -551,7 +561,6 @@ Studio::assignMidiProgramToInstrument(MidiByte program,
                                       int msb, int lsb,
                                       bool percussion)
 {
-    MidiDevice *midiDevice;
     std::vector<Device*>::iterator it;
     Rosegarden::InstrumentList::iterator iit;
     Rosegarden::InstrumentList instList;
@@ -568,12 +577,12 @@ Studio::assignMidiProgramToInstrument(MidiByte program,
     }
 
     // Pass one - search through all MIDI instruments looking for
-    // a match that we can re-use.  i.e. if we have a matching 
+    // a match that we can re-use.  i.e. if we have a matching
     // Program Change then we can use this Instrument.
     //
     for (it = m_devices.begin(); it != m_devices.end(); ++it)
     {
-        midiDevice = dynamic_cast<MidiDevice*>(*it);
+        MidiDevice* midiDevice = dynamic_cast<MidiDevice*>(*it);
 
         if (midiDevice && midiDevice->getDirection() == MidiDevice::Play)
         {
@@ -618,7 +627,7 @@ Studio::assignMidiProgramToInstrument(MidiByte program,
         }
     }
 
-    
+
     // Okay, if we've got this far and we have a new Instrument to use
     // then use it.
     //
@@ -645,14 +654,13 @@ Studio::assignMidiProgramToInstrument(MidiByte program,
 // assignment in the assignMidiProgramToInstrument() method
 // by invalidating the ProgramChange flag.
 //
-// This method sounds much more dramatic than it actually is - 
+// This method sounds much more dramatic than it actually is -
 // it could probably do with a rename.
 //
 //
 void
 Studio::unassignAllInstruments()
 {
-    MidiDevice *midiDevice;
     AudioDevice *audioDevice;
     std::vector<Device*>::iterator it;
     Rosegarden::InstrumentList::iterator iit;
@@ -661,7 +669,7 @@ Studio::unassignAllInstruments()
 
     for (it = m_devices.begin(); it != m_devices.end(); ++it)
     {
-        midiDevice = dynamic_cast<MidiDevice*>(*it);
+        MidiDevice* midiDevice = dynamic_cast<MidiDevice*>(*it);
 
         if (midiDevice)
         {
@@ -675,7 +683,7 @@ Studio::unassignAllInstruments()
                 {
                     (*iit)->setSendBankSelect(false);
                     (*iit)->setSendProgramChange(false);
-                    (*iit)->setNaturalChannel(channel);
+                    (*iit)->setNaturalMidiChannel(channel);
                     channel = ( channel + 1 ) % 16;
                     (*iit)->setFixedChannel();
                     // ??? This is a "reset" of the instrument.  It doesn't
@@ -683,8 +691,6 @@ Studio::unassignAllInstruments()
                     //     default values.
                     //(*iit)->sendChannelSetup();
 
-                    (*iit)->setSendPan(false);
-                    (*iit)->setSendVolume(false);
                     (*iit)->setPan(MidiMidValue);
                     (*iit)->setVolume(100);
 
@@ -709,12 +715,11 @@ Studio::unassignAllInstruments()
 void
 Studio::clearMidiBanksAndPrograms()
 {
-    MidiDevice *midiDevice;
     std::vector<Device*>::iterator it;
 
     for (it = m_devices.begin(); it != m_devices.end(); ++it)
     {
-        midiDevice = dynamic_cast<MidiDevice*>(*it);
+        MidiDevice* midiDevice = dynamic_cast<MidiDevice*>(*it);
 
         if (midiDevice)
         {
@@ -748,17 +753,17 @@ Device *
 Studio::getDevice(DeviceId id) const
 {
     //RG_DEBUG << "Studio[" << this << "]::getDevice(" << id << ")... ";
-    
+
     std::vector<Device*>::const_iterator it;
-    
+
     for (it = m_devices.begin(); it != m_devices.end(); ++it) {
-        
+
         // possibly fix a following seg.fault :
-        if( ! (*it) ){ 
+        if( ! (*it) ){
             RG_WARNING << "getDevice(): WARNING: (*it) is nullptr";
             continue;
         }
-        
+
         //RG_DEBUG << (*it)->getId();
 
         if ((*it)->getId() == id) {
@@ -766,31 +771,33 @@ Studio::getDevice(DeviceId id) const
             return (*it);
         }
     }
-    
+
     //RG_DEBUG << "NOT found";
-    
+
     return nullptr;
 }
 
 Device *
-Studio::getAudioDevice()
+Studio::getAudioDevice() const
 {
-    std::vector<Device*>::iterator it;
-
-    for (it = m_devices.begin(); it != m_devices.end(); ++it) {
-        if ((*it)->getType() == Device::Audio) return *it;
+    // For each Device
+    for (Device *device : m_devices) {
+        // Audio?  Return it.
+        if (device->getType() == Device::Audio)
+            return device;
     }
 
     return nullptr;
 }
 
 Device *
-Studio::getSoftSynthDevice()
+Studio::getSoftSynthDevice() const
 {
-    std::vector<Device*>::iterator it;
-
-    for (it = m_devices.begin(); it != m_devices.end(); ++it) {
-        if ((*it)->getType() == Device::SoftSynth) return *it;
+    // For each Device
+    for (Device *device : m_devices) {
+        // SoftSynth?  Return it.
+        if (device->getType() == Device::SoftSynth)
+            return device;
     }
 
     return nullptr;
@@ -799,14 +806,13 @@ Studio::getSoftSynthDevice()
 std::string
 Studio::getSegmentName(InstrumentId id)
 {
-    MidiDevice *midiDevice;
     std::vector<Device*>::iterator it;
     Rosegarden::InstrumentList::iterator iit;
     Rosegarden::InstrumentList instList;
 
     for (it = m_devices.begin(); it != m_devices.end(); ++it)
     {
-        midiDevice = dynamic_cast<MidiDevice*>(*it);
+        MidiDevice* midiDevice = dynamic_cast<MidiDevice*>(*it);
 
         if (midiDevice)
         {
@@ -835,12 +841,11 @@ Studio::getSegmentName(InstrumentId id)
 InstrumentId
 Studio::getAudioPreviewInstrument()
 {
-    AudioDevice *audioDevice;
     std::vector<Device*>::iterator it;
 
     for (it = m_devices.begin(); it != m_devices.end(); ++it)
     {
-        audioDevice = dynamic_cast<AudioDevice*>(*it);
+        AudioDevice* audioDevice = dynamic_cast<AudioDevice*>(*it);
 
         // Just the first one will do - we can make this more
         // subtle if we need to later.
@@ -863,7 +868,63 @@ Studio::haveMidiDevices() const
     }
     return false;
 }
-    
 
+Device *
+Studio::getFirstMIDIOutDevice() const
+{
+    // For each Device...
+    for (Device *device : m_devices) {
+        if (!device)
+            continue;
+
+        // MIDI Devices only.
+        if (device->getType() != Device::Midi)
+            continue;
+        // Output only.
+        if (!device->isOutput())
+            continue;
+
+        return device;
+    }
+
+    // Not found.
+    return nullptr;
 }
 
+InstrumentId
+Studio::getFirstMIDIInstrument() const
+{
+    const Device *device = getFirstMIDIOutDevice();
+    if (!device)
+        return SoftSynthInstrumentBase;
+
+    InstrumentList instruments = device->getPresentationInstruments();
+
+    if (!instruments.empty()) {
+        Instrument *instrument = instruments[0];
+        if (instrument)
+            return instrument->getId();
+    }
+
+    return SoftSynthInstrumentBase;
+}
+
+InstrumentId
+Studio::getAvailableMIDIInstrument(const Composition *composition) const
+{
+    const Device *device = getFirstMIDIOutDevice();
+    if (!device)
+        return SoftSynthInstrumentBase;
+
+    // Find an Instrument we can use.
+    const InstrumentId foundInstrumentID =
+            device->getAvailableInstrument(composition);
+
+    if (foundInstrumentID != NoInstrument)
+        return foundInstrumentID;
+
+    return SoftSynthInstrumentBase;
+}
+
+
+}
