@@ -3,7 +3,7 @@
 /*
     Rosegarden
     A MIDI and audio sequencer and musical notation editor.
-    Copyright 2000-2018 the Rosegarden development team.
+    Copyright 2000-2024 the Rosegarden development team.
 
     Other copyrights also apply to some parts of this work.  Please
     see the AUTHORS file and individual file headers for details.
@@ -20,19 +20,16 @@
 
 namespace Rosegarden
 {
-    // Time in internal units (pulses).  Timebase is 960ppq.
+    // Time in MIDI ticks (aka pulses as in ppqn, pulses per quarter note).
+    // ??? rename: MidiTicks
     typedef long timeT;
 
-    // Time used in the rewriter to represent tupled notes exactly.
-    // It doesn't have a fixed conversion to timeT, instead each bar
-    // figures out a scaling that allows all tupleting in it to be
-    // represented exactly.
-    typedef timeT stretchedTimeT;
-
-    // Time in 256th notes; equivalently, 4 times the time in smallest
-    // notes.  We need the unit to be 1/4 of the smallest note so we
-    // can represent dotted and double-dotted notes exactly.
-    typedef int Num256ths;
+    // Looks like there are various 960's scattered throughout the system.
+    // But 960ppq appears to be the standard for Rosegarden notation anyway.
+    // The sequencer deals in RealTime (nanosecond resolution) but is limited
+    // by the timer resolution.  See AlsaDriver::setCurrentTimer().
+    // 480 is used for MIDI file export.  See MidiFile::convertToMidi().
+    constexpr timeT timebase = 960;  // PPQ, PPQN, TPQN
 }
 
-#endif /* ifndef RG_TIMET_H */
+#endif
