@@ -3,11 +3,11 @@
 /*
     Rosegarden
     A MIDI and audio sequencer and musical notation editor.
-    Copyright 2000-2018 the Rosegarden development team.
- 
+    Copyright 2000-2024 the Rosegarden development team.
+
     Other copyrights also apply to some parts of this work.  Please
     see the AUTHORS file and individual file headers for details.
- 
+
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
     published by the Free Software Foundation; either version 2 of the
@@ -31,17 +31,12 @@
 #include "base/Studio.h"
 #include "base/Track.h"
 #include "gui/editors/notation/NotationStrings.h"
+
 #include <QFile>
-#include <QObject>
-#include <QString>
-#include <QStringList>
 #include <QTextStream>
+
 #include <string>
 #include <vector>
-#include <iostream>
-
-using std::vector;
-using std::string;
 
 
 namespace Rosegarden
@@ -51,8 +46,7 @@ using namespace BaseProperties;
 using namespace Accidentals;
 using namespace Marks;
 
-RG21Loader::RG21Loader(Studio *studio)
-        :
+RG21Loader::RG21Loader(Studio *studio) :
         m_stream(nullptr),
         m_studio(studio),
         m_composition(nullptr),
@@ -60,15 +54,24 @@ RG21Loader::RG21Loader(Studio *studio)
         m_currentSegmentTime(0),
         m_currentSegmentNb(0),
         m_currentClef(Clef::Treble),
+        m_currentKey(),
         m_currentInstrumentId(MidiInstrumentBase),
+        m_indicationsExtant(),
         m_inGroup(false),
+        m_groupId(0),
+        m_groupType(),
+        m_groupStartTime(0),
+        m_groupTupledLength(0),
+        m_groupTupledCount(0),
+        m_groupUntupledLength(0),
+        m_groupUntupledCount(0),
         m_tieStatus(0),
+        m_currentLine(),
+        m_currentStaffName(),
+        m_tokens(),
         m_nbStaves(0)
 {}
 
-RG21Loader::~RG21Loader()
-{}
-
 bool RG21Loader::parseClef()
 {
     if (m_tokens.count() != 3 || !m_currentSegment)
@@ -147,7 +150,7 @@ bool RG21Loader::parseChordItem()
     /*int nbNotes   = (*i).toInt();*/
     ++i;
 
-    vector<string> marks = convertRG21ChordMods(chordMods);
+    StringVector marks = convertRG21ChordMods(chordMods);
 
     // now get notes
     for (;i != m_tokens.end(); ++i) {
@@ -322,7 +325,7 @@ bool RG21Loader::parseGroupStart()
 
         RG_DEBUG
         << "RG21Loader::parseGroupStart: WARNING: Unknown group type "
-        << m_groupType << ", ignoring" << endl;
+        << m_groupType << ", ignoring";
         m_inGroup = false;
     }
 
@@ -344,7 +347,7 @@ bool RG21Loader::parseIndicationStart()
         if (m_tieStatus != 0) {
             RG_DEBUG
             << "RG21Loader:: parseIndicationStart: WARNING: Found tie within "
-            << "tie, ignoring" << endl;
+            << "tie, ignoring";
             return true;
         }
         // m_tieStatus = 1;
@@ -379,7 +382,7 @@ bool RG21Loader::parseIndicationStart()
 
         Indication indication(indicationType, 0);
         Event *e = indication.getAsEvent(indicationTime);
-        e->setMaybe<Int>("indicationId", indicationId);
+        e->setMaybe<Int>(PropertyName("indicationId"), indicationId);
         setGroupProperties(e);
         m_indicationsExtant[indicationId] = e;
 
@@ -389,8 +392,8 @@ bool RG21Loader::parseIndicationStart()
 
         m_currentSegment->insert(e);
 
-        RG_DEBUG << "rg21io: Indication start: it's a real indication; id is " << indicationId << ", event is:";
-        e->dump(std::cerr);
+        RG_DEBUG << "rg21io: parseIndicationStart(): it's a real indication; id is " << indicationId << ", event is:";
+        RG_DEBUG << e;
 
     }
 
@@ -418,7 +421,7 @@ void RG21Loader::closeIndication()
     indicationEvent->set
     <Int>
     //!!!	(Indication::IndicationDurationPropertyName,
-    ("indicationduration",
+    (PropertyName("indicationduration"),
      m_currentSegmentTime - indicationEvent->getAbsoluteTime());
 }
 
@@ -427,8 +430,8 @@ void RG21Loader::closeGroup()
     if (m_groupType == GROUP_TYPE_TUPLED) {
 
         Segment::iterator i = m_currentSegment->end();
-        vector<Event *> toInsert;
-        vector<Segment::iterator> toErase;
+        std::vector<Event *> toInsert;
+        std::vector<Segment::iterator> toErase;
 
         if (i != m_currentSegment->begin()) {
 
@@ -457,8 +460,7 @@ void RG21Loader::closeGroup()
                 << ", new absolute time = "
                 << (absoluteTime + intended - offset)
                 << ", new duration = "
-                << (prev - absoluteTime)
-                << endl;
+                << (prev - absoluteTime);
 
                 absoluteTime = absoluteTime + intended - offset;
                 Event *e(new Event(**i, absoluteTime, prev - absoluteTime));
@@ -483,7 +485,7 @@ void RG21Loader::closeGroup()
                 if ((*i)->isa(Indication::EventType)) {
                     long indicationId = 0;
                     if ((*i)->get
-                            <Int>("indicationId", indicationId)) {
+                            <Int>(PropertyName("indicationId"), indicationId)) {
                         EventIdMap::iterator ei =
                             m_indicationsExtant.find(indicationId);
                         if (ei != m_indicationsExtant.end()) {
@@ -526,7 +528,7 @@ bool RG21Loader::parseBarType()
     if (staffNo > 0) {
         RG_DEBUG
         << "RG21Loader::parseBarType: We don't support different time\n"
-        << "signatures on different staffs; disregarding time signature for staff " << staffNo << endl;
+        << "signatures on different staffs; disregarding time signature for staff " << staffNo;
         return true;
     }
 
@@ -592,10 +594,10 @@ timeT RG21Loader::convertRG21Duration(QStringList::Iterator& i)
         Note n(NotationStrings::getNoteForName(durationString));
         return n.getDuration();
 
-    } catch (NotationStrings::MalformedNoteName m) {
+    } catch (const NotationStrings::MalformedNoteName &m) {
 
         RG_DEBUG << "RG21Loader::convertRG21Duration: Bad duration: "
-        << durationString << endl;
+        << durationString;
         return 0;
     }
 
@@ -630,7 +632,7 @@ void RG21Loader::closeSegment()
     }
 }
 
-long RG21Loader::convertRG21Pitch(long pitch, int noteModifier)
+long RG21Loader::convertRG21Pitch(long rg21pitch, int noteModifier)
 {
     Accidental accidental =
         (noteModifier & ModSharp) ? Sharp :
@@ -638,7 +640,7 @@ long RG21Loader::convertRG21Pitch(long pitch, int noteModifier)
         (noteModifier & ModNatural) ? Natural : NoAccidental;
 
     long rtn = Pitch::getPerformancePitchFromRG21Pitch
-               (pitch, accidental, m_currentClef, m_currentKey);
+               (rg21pitch, accidental, m_currentClef, m_currentKey);
 
     return rtn;
 }
@@ -663,7 +665,11 @@ bool RG21Loader::readNextLine()
             continue; // skip comments
         }
 
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
+        m_tokens = m_currentLine.split(' ', Qt::SkipEmptyParts);
+#else
         m_tokens = m_currentLine.split(' ', QString::SkipEmptyParts);
+#endif
 
     } while (inComment);
 
@@ -774,22 +780,27 @@ bool RG21Loader::load(const QString &fileName, Composition &comp)
     delete m_stream;
     m_stream = nullptr;
 
+    // No tracks?  Then we failed to load this file.
+    if (m_composition->getNbTracks() == 0)
+        return false;
+
     return true;
 }
 
-vector<string> RG21Loader::convertRG21ChordMods(int chordMods)
+RG21Loader::StringVector
+RG21Loader::convertRG21ChordMods(int chordMod)
 {
-    vector<string> marks;
+    StringVector marks;
 
     // bit laborious!
-    if (chordMods & ModDot)    marks.push_back(Staccato);
-    if (chordMods & ModLegato) marks.push_back(Tenuto);
-    if (chordMods & ModAccent) marks.push_back(Accent);
-    if (chordMods & ModSfz)    marks.push_back(Sforzando);
-    if (chordMods & ModRfz)    marks.push_back(Rinforzando);
-    if (chordMods & ModTrill)  marks.push_back(Trill);
-    if (chordMods & ModTurn)   marks.push_back(Turn);
-    if (chordMods & ModPause)  marks.push_back(Pause);
+    if (chordMod & ModDot)    marks.push_back(Staccato);
+    if (chordMod & ModLegato) marks.push_back(Tenuto);
+    if (chordMod & ModAccent) marks.push_back(Accent);
+    if (chordMod & ModSfz)    marks.push_back(Sforzando);
+    if (chordMod & ModRfz)    marks.push_back(Rinforzando);
+    if (chordMod & ModTrill)  marks.push_back(Trill);
+    if (chordMod & ModTurn)   marks.push_back(Turn);
+    if (chordMod & ModPause)  marks.push_back(Pause);
 
     return marks;
 }
