@@ -3,7 +3,7 @@
 /*
     Rosegarden
     A MIDI and audio sequencer and musical notation editor.
-    Copyright 2000-2018 the Rosegarden development team.
+    Copyright 2000-2024 the Rosegarden development team.
 
     Other copyrights also apply to some parts of this work.  Please
     see the AUTHORS file and individual file headers for details.
@@ -15,6 +15,9 @@
     COPYING included with this distribution for more information.
 */
 
+#define RG_MODULE_STRING "[NotationScene]"
+#define RG_NO_DEBUG_PRINT 1
+
 #include "NotationScene.h"
 
 #include "base/Segment.h"
@@ -59,7 +62,7 @@ static int instanceCount = 0;
 NotationScene::NotationScene() :
     m_widget(nullptr),
     m_document(nullptr),
-    m_properties(nullptr),
+    m_properties(),
     m_notePixmapFactory(nullptr),
     m_notePixmapFactorySmall(nullptr),
     m_clefKeyContext(new ClefKeyContext),
@@ -84,10 +87,11 @@ NotationScene::NotationScene() :
     m_sceneIsEmpty(false),
     m_showRepeated(false),
     m_editRepeated(false),
-    m_haveInittedCurrentStaff(false)
+    m_haveInittedCurrentStaff(false),
+    m_previewNoteStaff(nullptr)
 {
     QString prefix(QString("NotationScene%1::").arg(instanceCount++));
-    m_properties = new NotationProperties(qstrtostr(prefix));
+    m_properties.reset(new NotationProperties(qstrtostr(prefix)));
 
 //    qRegisterMetaType<NotationMouseEvent>("Rosegarden::NotationMouseEvent");
 
@@ -117,7 +121,7 @@ NotationScene::~NotationScene()
     delete m_clefKeyContext;
 
     for (unsigned int i = 0; i < m_staffs.size(); ++i) delete m_staffs[i];
-    
+
     for (std::vector<Segment *>::iterator it = m_clones.begin();
          it != m_clones.end(); ++it) {
         delete (*it);
@@ -162,7 +166,7 @@ NotationScene::getFontName() const
 }
 
 void
-NotationScene::setFontName(QString name)
+NotationScene::setFontName(const QString& name)
 {
     if (name == getFontName()) return;
     setNotePixmapFactories(name, getFontSize());
@@ -206,11 +210,13 @@ NotationScene::setHSpacing(int spacing)
     }
 }
 
+/* unused
 int
 NotationScene::getLeftGutter() const
 {
     return m_leftGutter;
 }
+*/
 
 void
 NotationScene::setLeftGutter(int gutter)
@@ -303,7 +309,7 @@ NotationScene::setStaffs(RosegardenDocument *document,
         // External segments and clones are now mixed inside m_segments
     } else {
         m_segments = m_externalSegments;
-        // No clone in that case 
+        // No clone in that case
     }
 
 
@@ -352,7 +358,7 @@ NotationScene::setStaffs(RosegardenDocument *document,
     m_visibleStaffs = trackIds.size();
 
     m_clefKeyContext->setSegments(this);
-    
+
     // Remember the names of the tracks
     for (std::set<TrackId>::iterator i = trackIds.begin();
          i != trackIds.end(); ++i) {
@@ -379,17 +385,17 @@ NotationScene::setStaffs(RosegardenDocument *document,
     }
 
 
-    connect(CommandHistory::getInstance(), SIGNAL(commandExecuted()),
-            this, SLOT(slotCommandExecuted()));
+    connect(CommandHistory::getInstance(), &CommandHistory::commandExecuted,
+            this, &NotationScene::slotCommandExecuted);
 }
 
 
 void
 NotationScene::createClonesFromRepeatedSegments()
 {
-    const Segment::Participation participation = 
+    const Segment::Participation participation =
         m_editRepeated ? Segment::editableClone : Segment::justForShow;
-    
+
     // Create clones (if needed)
     for (std::vector<Segment *>::iterator it = m_externalSegments.begin();
         it != m_externalSegments.end(); ++it) {
@@ -528,30 +534,50 @@ NotationScene::getNextStaffOnTrack()
     return getNextStaffHorizontally(1, false);
 }
 
+NotationStaff *
+NotationScene::getStaffBySegmentMarking(const QString& marking) const
+{
+    for (unsigned int i=0; i<m_staffs.size(); ++i) {
+        NotationStaff* staff = m_staffs[i];
+        QString staffMarking = staff->getMarking();
+        if (staffMarking == marking) {
+            return staff;
+        }
+    }
+    return nullptr;
+}
+
 NotationStaff *
 NotationScene::getStaffbyTrackAndTime(const Track *track, timeT targetTime)
 {
     // Prepare a fallback: If this is the right track but no staff
     // includes time t, we'll return the fallback instead.  We
     // don't try to find the best fallback.
-    bool haveFallback = false;
     NotationStaff * fallback = nullptr;
+    timeT minTime = 1.0e10;
     for (unsigned int i = 0; i < m_staffs.size(); ++i) {
         if (m_staffs[i]->getSegment().getTrack() == track->getId()) {
             if(m_staffs[i]->includesTime(targetTime)) {
                 return m_staffs[i];
             } else {
-                haveFallback = true;
-                fallback = m_staffs[i];
+                // find the closest staff to the targetTime
+                timeT dt1 = abs(targetTime - m_staffs[i]->getStartTime());
+                if (dt1 < minTime) {
+                    minTime = dt1;
+                    fallback = m_staffs[i];
+                }
+                timeT dt2 = abs(targetTime - m_staffs[i]->getEndTime());
+                if (dt2 < minTime) {
+                    minTime = dt2;
+                    fallback = m_staffs[i];
+                }
             }
         }
     }
     // We found segments on the track, but none that include time
     // t.  In this circumstance, we still want to return a staff
     // so return the fallback.
-    if (haveFallback) { return fallback; }
-    
-    return nullptr;
+    return fallback;
 }
 
 // @params
@@ -630,13 +656,13 @@ NotationScene::initCurrentStaffIndex()
     // otherwise we'll annoy the user.
     if (m_haveInittedCurrentStaff) { return; }
     m_haveInittedCurrentStaff = true;
-    
+
     // Can't do much if we have no staffs.
     if (m_staffs.empty()) { return; }
 
     Composition &composition = m_document->getComposition();
     timeT targetTime = composition.getPosition();
-    
+
     // Try the globally selected track (which we may not even include
     // any segments from)
     {
@@ -652,7 +678,7 @@ NotationScene::initCurrentStaffIndex()
     {
         // Careful, m_minTrack is an int indicating position, not a
         // TrackId, and must be converted.
-        const Track *track = 
+        const Track *track =
             composition.getTrackByPosition(m_minTrack);
         NotationStaff *staff = getStaffbyTrackAndTime(track, targetTime);
         if (staff) {
@@ -660,7 +686,7 @@ NotationScene::initCurrentStaffIndex()
             return;
         }
     }
-    
+
     // We shouldn't reach here.
     RG_WARNING << "Argh! Failed to find a staff!";
 }
@@ -718,7 +744,7 @@ NotationScene::setupMouseEvent(QPointF scenePos, Qt::MouseButtons buttons,
                                Qt::KeyboardModifiers modifiers,
                                NotationMouseEvent &nme)
 {
-    Profiler profiler("NotationScene::setupMouseEvent");
+    //Profiler profiler("NotationScene::setupMouseEvent");
 
     double sx = scenePos.x();
     int sy = lrint(scenePos.y());
@@ -768,6 +794,7 @@ NotationScene::setupMouseEvent(QPointF scenePos, Qt::MouseButtons buttons,
     // clicking on something specific
 
     const QList<QGraphicsItem *> collisions = items(scenePos);
+    //RG_DEBUG << "setupMouseEvent collisions:" << collisions.size();
 
     NotationElement *clickedNote = nullptr;
     NotationElement *clickedVagueNote = nullptr;
@@ -778,19 +805,28 @@ NotationScene::setupMouseEvent(QPointF scenePos, Qt::MouseButtons buttons,
 
         NotationElement *element = NotationElement::getNotationElement(*i);
         if (!element) continue;
+        RG_DEBUG << "setupMouseEvent event:" << *(element->event());
 
         // #957364 (Notation: Hard to select upper note in chords of
         // seconds) -- adjust x-coord for shifted note head
 
         double cx = element->getSceneX();
-        int nbw = 10;
 
-        nbw = m_notePixmapFactory->getNoteBodyWidth();
+        int nbw = m_notePixmapFactory->getNoteBodyWidth();
         bool shifted = false;
 
         if (element->event()->get<Bool>
             (m_properties->NOTE_HEAD_SHIFTED, shifted) && shifted) {
-            cx += nbw;
+            bool stemUp = false;
+            element->event()->get<Bool>(m_properties->VIEW_LOCAL_STEM_UP,
+                                        stemUp);
+            RG_DEBUG << "setupMouseEvent shift" << cx << nbw << stemUp;
+
+            if (stemUp) {
+                cx += nbw;
+            } else {
+                cx -= nbw;
+            }
         }
 
         if (element->isNote() && haveClickHeight) {
@@ -802,28 +838,35 @@ NotationScene::setupMouseEvent(QPointF scenePos, Qt::MouseButtons buttons,
 
                 if (eventHeight == nme.height) {
 
+                    RG_DEBUG << "setupMouseEvent t1" << nme.sceneX << cx <<
+                        nbw;
                     if (!clickedNote &&
                         nme.sceneX >= cx &&
                         nme.sceneX <= cx + nbw) {
+                        RG_DEBUG << "setupMouseEvent cn1" << element;
                         clickedNote = element;
                     } else if (!clickedVagueNote &&
                                nme.sceneX >= cx - 2 &&
                                nme.sceneX <= cx + nbw + 2) {
+                        RG_DEBUG << "setupMouseEvent cnv1" << element;
                         clickedVagueNote = element;
                     }
 
                 } else if (eventHeight - 1 == nme.height ||
                            eventHeight + 1 == nme.height) {
                     if (!clickedVagueNote) {
+                        RG_DEBUG << "setupMouseEvent cv2" << element;
                         clickedVagueNote = element;
                     }
                 }
             }
         } else if (!element->isNote()) {
             if (!clickedNonNote) {
+                RG_DEBUG << "setupMouseEvent cn2" << element;
                 clickedNonNote = element;
             }
         }
+        RG_DEBUG << "setupMouseEvent" << clickedNote;
     }
 
     nme.exact = false;
@@ -852,14 +895,18 @@ NotationScene::setupMouseEvent(QPointF scenePos, Qt::MouseButtons buttons,
                    << ", height = " << nme.height;*/
 }
 
+void
+NotationScene::slotMouseLeavesView()
+{
+    clearPreviewNote();
+}
+
 void
 NotationScene::mousePressEvent(QGraphicsSceneMouseEvent *e)
 {
     NotationMouseEvent nme;
     setupMouseEvent(e, nme);
-    ///! Warning, this short-circuits NotationView::setCurrentStaff...
-    if (nme.staff) setCurrentStaff(nme.staff);
-    emit mousePressed(&nme);
+    m_widget->dispatchMousePress(&nme);
 }
 
 void
@@ -867,7 +914,7 @@ NotationScene::mouseMoveEvent(QGraphicsSceneMouseEvent *e)
 {
     NotationMouseEvent nme;
     setupMouseEvent(e, nme);
-    emit mouseMoved(&nme);
+    m_widget->dispatchMouseMove(&nme);
 }
 
 void
@@ -875,7 +922,7 @@ NotationScene::mouseReleaseEvent(QGraphicsSceneMouseEvent *e)
 {
     NotationMouseEvent nme;
     setupMouseEvent(e, nme);
-    emit mouseReleased(&nme);
+    m_widget->dispatchMouseRelease(&nme);
 }
 
 void
@@ -883,7 +930,7 @@ NotationScene::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *e)
 {
     NotationMouseEvent nme;
     setupMouseEvent(e, nme);
-    emit mouseDoubleClicked(&nme);
+    m_widget->dispatchMouseDoubleClick(&nme);
 }
 
 void
@@ -892,12 +939,12 @@ NotationScene::wheelEvent(QGraphicsSceneWheelEvent *e)
     if (m_widget->getCurrentTool()->needsWheelEvents()) {
         NotationMouseEvent nme;
         setupMouseEvent(e, nme);
-        emit wheelTurned(e->delta(), &nme);
+        m_widget->dispatchWheelTurned(e->delta(), &nme);
         e->accept();    // Don't pass the event to the view
     }
 }
 
-void 
+void
 NotationScene::keyPressEvent(QKeyEvent * keyEvent)
 {
     processKeyboardEvent(keyEvent);
@@ -921,13 +968,15 @@ NotationScene::processKeyboardEvent(QKeyEvent * keyEvent)
         QPoint pos = m_widget->getView()->viewport()->mapFromGlobal(globalPos);
         QPointF scenePos = m_widget->getView()->mapToScene(pos);
 
-        // Create a NotationMouseEvent related to the QKeyEvent
+        // Create a NotationMouseEvent related to the QKeyEvent.
+        // Use queryKeyboardModifiers() rather than keyboardModifiers()
+        // to ensure the current value of modifiers is read.
         NotationMouseEvent nme;
         setupMouseEvent(scenePos, QApplication::mouseButtons(),
-                        QApplication::keyboardModifiers(), nme);
+                        QApplication::queryKeyboardModifiers(), nme);
 
         // Handle it as a mouse event
-        emit mouseMoved(&nme);
+        m_widget->dispatchMouseMove(&nme);;
     }
 }
 
@@ -1021,7 +1070,7 @@ NotationScene::setPageMode(StaffLayout::PageMode mode)
     if (!m_printMode) {
         updateView();
         slotSetInsertCursorPosition(getInsertionTime(), false, false);
-        slotSetPointerPosition(getDocument()->getComposition().getPosition(), false);
+        slotSetPointerPosition(RosegardenDocument::currentDocument->getComposition().getPosition(), false);
     }
 */
 }
@@ -1040,10 +1089,11 @@ NotationScene::timeSignatureChanged(const Composition *c)
 }
 
 timeT
-NotationScene::getInsertionTime() const
+NotationScene::getInsertionTime(bool allowEndTime) const
 {
     if (!m_document) return 0;
-    return snapTimeToNoteBoundary(m_document->getComposition().getPosition());
+    return snapTimeToNoteBoundary(m_document->getComposition().getPosition(),
+                                  allowEndTime);
 }
 
 NotationScene::CursorCoordinates
@@ -1068,7 +1118,7 @@ NotationScene::getCursorCoordinates(timeT t) const
         currentStaff = m_staffs[m_currentStaff];
     }
 
-    timeT snapped = snapTimeToNoteBoundary(t);
+    timeT snapped = snapTimeToNoteBoundary(t, true);
 
     double x = m_hlayout->getXForTime(t);
     double sx = m_hlayout->getXForTimeByEvent(snapped);
@@ -1102,7 +1152,7 @@ NotationScene::getCursorCoordinates(timeT t) const
 }
 
 timeT
-NotationScene::snapTimeToNoteBoundary(timeT t) const
+NotationScene::snapTimeToNoteBoundary(timeT t, bool allowEndTime) const
 {
     NotationStaff *s = nullptr;
     if (m_currentStaff < (int)m_staffs.size()) {
@@ -1115,7 +1165,13 @@ NotationScene::snapTimeToNoteBoundary(timeT t) const
     if (i == v->end()) i = v->begin();
     if (i == v->end()) return t;
 
-    return (*i)->getViewAbsoluteTime();
+    timeT eventTime = (*i)->getViewAbsoluteTime();
+    if (allowEndTime && t > eventTime) {
+        timeT staffEnd = s->getEndTime();
+        if (t > staffEnd) t = staffEnd;
+        return t;
+    }
+    return eventTime;
 }
 
 void
@@ -1176,8 +1232,10 @@ NotationScene::checkUpdate()
 void
 NotationScene::dumpVectors()
 {
+    RG_DEBUG << "dumpVectors begin";
     for (unsigned int i=0; i<m_externalSegments.size(); ++i) {
-        RG_DEBUG << "extern" << i << ":" << m_externalSegments[i];
+        RG_DEBUG << "extern" << i << ":" << m_externalSegments[i] <<
+            m_externalSegments[i]->getLabel().c_str();
         if (m_externalSegments[i]->isTmp()) RG_DEBUG << " TMP";
         if (m_externalSegments[i]->isLinked()) RG_DEBUG << " LINKED";
         if (m_externalSegments[i]->isTrulyLinked()) RG_DEBUG << " TRULYLINKED";
@@ -1191,14 +1249,17 @@ NotationScene::dumpVectors()
                   << "endMrkr=" << m_clones[i]->getEndMarkerTime();
     }
     for (unsigned int i=0; i<m_segments.size(); ++i) {
-        RG_DEBUG << "segment" << i << ":" << m_segments[i];
+        RG_DEBUG << "segment" << i << ":" << m_segments[i] <<
+            m_segments[i]->getLabel().c_str();
         if (m_segments[i]->isTmp()) RG_DEBUG << " TMP";
         m_segments[i]->dumpObservers();
     }
     for (unsigned int i=0; i<m_staffs.size(); ++i) {
-        RG_DEBUG << "staff" << i << ":" << &m_staffs[i]->getSegment();
+        RG_DEBUG << "staff" << i << ":" << &m_staffs[i]->getSegment() <<
+            m_staffs[i]->getSegment().getLabel().c_str();
         if (m_staffs[i]->getSegment().isTmp()) RG_DEBUG << " TMP";
     }
+    RG_DEBUG << "dumpVectors end";
 }
 
 void
@@ -1207,9 +1268,12 @@ NotationScene::segmentRemoved(const Composition *c, Segment *s)
     NOTATION_DEBUG << "NotationScene::segmentRemoved(" << c << "," << s << ")";
     if (!m_document || !c || (c != &m_document->getComposition())) return;
 
+    std::vector<NotationStaff *>::iterator staffToDelete = m_staffs.end();
+
     for (std::vector<NotationStaff *>::iterator i = m_staffs.begin();
          i != m_staffs.end(); ++i) {
         if (s == &(*i)->getSegment()) {
+            staffToDelete = i;
 
             m_segmentsDeleted.push_back(s); // Remember segment to be deleted
 
@@ -1217,8 +1281,9 @@ NotationScene::segmentRemoved(const Composition *c, Segment *s)
             // the whole scene is going to be deleted then restored (from
             // NotationView) and to continue processing at best is useless and
             // at the worst may cause a crash when the segment is deleted.
-            disconnect(CommandHistory::getInstance(), SIGNAL(commandExecuted()),
-                       this, SLOT(slotCommandExecuted()));
+            disconnect(CommandHistory::getInstance(),
+                           &CommandHistory::commandExecuted,
+                       this, &NotationScene::slotCommandExecuted);
             suspendLayoutUpdates();   // Useful ???
 
             if (m_segmentsDeleted.size() == m_externalSegments.size()) {
@@ -1234,6 +1299,23 @@ NotationScene::segmentRemoved(const Composition *c, Segment *s)
             break;
         }
     }
+    // the sceneNeedsRebuilding signal will cause the scene to be
+    // rebuilt. However this uses a queued connection so the pointer
+    // update after undo is done before this so we must remove the
+    // deleted staff here.
+    if (staffToDelete != m_staffs.end()) {
+        NotationStaff* staff = *staffToDelete;
+        if (m_previewNoteStaff == staff) {
+            // clear the preview on the staff to be deleted
+            clearPreviewNote();
+        }
+        delete staff;
+        m_staffs.erase(staffToDelete);
+    }
+
+    // Redo the layouts so that there aren't any stray pointers
+    // to the removed staff.
+    layout(nullptr, 0, 0);
 }
 
 void
@@ -1252,8 +1334,9 @@ NotationScene::segmentRepeatChanged(const Composition *c, Segment *s, bool)
             // Therefore the whole scene is going to be deleted then restored
             // (from NotationView) and there is no point to continue processing
             // signals from the current NotationScene.
-            disconnect(CommandHistory::getInstance(), SIGNAL(commandExecuted()),
-                       this, SLOT(slotCommandExecuted()));
+            disconnect(CommandHistory::getInstance(),
+                           &CommandHistory::commandExecuted,
+                       this, &NotationScene::slotCommandExecuted);
             suspendLayoutUpdates();
             m_finished = true;    // Stop further processing from this scene
 
@@ -1280,8 +1363,9 @@ NotationScene::segmentRepeatEndChanged(const Composition *c, Segment *s, timeT)
             // Therefore the whole scene is going to be deleted then restored
             // (from NotationView) and to continue processing at best is
             // useless and at worst may cause a crash related to deleted clones.
-            disconnect(CommandHistory::getInstance(), SIGNAL(commandExecuted()),
-                       this, SLOT(slotCommandExecuted()));
+            disconnect(CommandHistory::getInstance(),
+                           &CommandHistory::commandExecuted,
+                       this, &NotationScene::slotCommandExecuted);
             suspendLayoutUpdates();
             m_finished = true;    // Stop further processing from this scene
 
@@ -1308,8 +1392,9 @@ NotationScene::segmentStartChanged(const Composition *c, Segment *s, timeT)
             // Therefore the whole scene is going to be deleted then restored
             // (from NotationView) and to continue processing at best is
             // useless and at worst may cause a crash related to deleted clones.
-            disconnect(CommandHistory::getInstance(), SIGNAL(commandExecuted()),
-                       this, SLOT(slotCommandExecuted()));
+            disconnect(CommandHistory::getInstance(),
+                           &CommandHistory::commandExecuted,
+                       this, &NotationScene::slotCommandExecuted);
             suspendLayoutUpdates();
             m_finished = true;    // Stop further processing from this scene
 
@@ -1336,8 +1421,9 @@ NotationScene::segmentEndMarkerChanged(const Composition *c, Segment *s, bool)
             // Therefore the whole scene is going to be deleted then restored
             // (from NotationView) and to continue processing at best is
             // useless and at worst may cause a crash related to deleted clones.
-            disconnect(CommandHistory::getInstance(), SIGNAL(commandExecuted()),
-                       this, SLOT(slotCommandExecuted()));
+            disconnect(CommandHistory::getInstance(),
+                           &CommandHistory::commandExecuted,
+                       this, &NotationScene::slotCommandExecuted);
             suspendLayoutUpdates();
             m_finished = true;    // Stop further processing from this scene
 
@@ -1364,15 +1450,16 @@ NotationScene::trackChanged(const Composition *c, Track *t)
         // Is the segment part of the changed track ?
         if ((*i)->getTrack() == trackId) {
 
-            // The scene needs a rebuild only if what has changed is the 
+            // The scene needs a rebuild only if what has changed is the
             // name of the track
             if (t->getLabel() == m_trackLabels[trackId]) break;
 
             // The whole scene is going to be deleted then restored
             // (from NotationView). To continue processing at best is
             // useless and at worst may cause a crash related to deleted clones.
-            disconnect(CommandHistory::getInstance(), SIGNAL(commandExecuted()),
-                       this, SLOT(slotCommandExecuted()));
+            disconnect(CommandHistory::getInstance(),
+                           &CommandHistory::commandExecuted,
+                       this, &NotationScene::slotCommandExecuted);
             suspendLayoutUpdates();
             m_finished = true;    // Stop further processing from this scene
 
@@ -1431,7 +1518,7 @@ NotationScene::positionStaffs()
 
         QVariant fv = settings.value("textfont", defaultFont);
         QFont font(defaultFont);
-        if (fv.canConvert(QVariant::Font)) font = fv.value<QFont>();
+        if (fv.canConvert<QFont>()) font = fv.value<QFont>();
 
         font.setPixelSize(m_notePixmapFactory->getSize() * 5);
         QFontMetrics metrics(font);
@@ -1443,7 +1530,7 @@ NotationScene::positionStaffs()
             m_title->setDefaultTextColor(Qt::black);
             addItem(m_title);
             m_title->setFont(font);
-            m_title->setPos(m_leftGutter + pageWidth / 2 - metrics.width(title) / 2,
+            m_title->setPos(m_leftGutter + pageWidth / 2 - metrics.boundingRect(title).width() / 2,
                             20 + topMargin / 4 + metrics.ascent());
             m_title->show();
             titleHeight += metrics.height() * 3 / 2 + topMargin / 4;
@@ -1459,7 +1546,7 @@ NotationScene::positionStaffs()
             m_subtitle->setDefaultTextColor(Qt::black);
             addItem(m_subtitle);
             m_subtitle->setFont(font);
-            m_subtitle->setPos(m_leftGutter + pageWidth / 2 - metrics.width(subtitle) / 2,
+            m_subtitle->setPos(m_leftGutter + pageWidth / 2 - metrics.boundingRect(subtitle).width() / 2,
                                20 + titleHeight + metrics.ascent());
             m_subtitle->show();
             titleHeight += metrics.height() * 3 / 2;
@@ -1472,7 +1559,7 @@ NotationScene::positionStaffs()
             m_composer->setDefaultTextColor(Qt::black);
             addItem(m_composer);
             m_composer->setFont(font);
-            m_composer->setPos(m_leftGutter + pageWidth - metrics.width(composer) - leftMargin,
+            m_composer->setPos(m_leftGutter + pageWidth - metrics.boundingRect(composer).width() - leftMargin,
                                20 + titleHeight + metrics.ascent());
             m_composer->show();
             titleHeight += metrics.height() * 3 / 2;
@@ -1681,7 +1768,7 @@ NotationScene::positionStaffs()
 void
 NotationScene::layoutAll()
 {
-    Profiler profiler("NotationScene::layoutAll", true);
+    //Profiler profiler("NotationScene::layoutAll", true);
     layout(nullptr, 0, 0);
 }
 
@@ -1689,7 +1776,7 @@ void
 NotationScene::layout(NotationStaff *singleStaff,
                       timeT startTime, timeT endTime)
 {
-    Profiler profiler("NotationScene::layout", true);
+    //Profiler profiler("NotationScene::layout", true);
     NOTATION_DEBUG << "NotationScene::layout: from " << startTime << " to " << endTime;
 
     bool full = (singleStaff == nullptr && startTime == endTime);
@@ -1698,7 +1785,7 @@ NotationScene::layout(NotationStaff *singleStaff,
 
     if (full) {
 
-        Profiler profiler("NotationScene::layout: Reset layouts for full scan", true);
+        //Profiler profiler("NotationScene::layout: Reset layouts for full scan", true);
 
         m_hlayout->reset();
         m_vlayout->reset();
@@ -1720,7 +1807,7 @@ NotationScene::layout(NotationStaff *singleStaff,
     NOTATION_DEBUG << "overall start time =" << startTime << ", end time =" << endTime;
 
     {
-        Profiler profiler("NotationScene::layout: Scan layouts", true);
+        //Profiler profiler("NotationScene::layout: Scan layouts", true);
     for (unsigned int i = 0; i < m_staffs.size(); ++i) {
 
         NotationStaff *staff = m_staffs[i];
@@ -1775,7 +1862,7 @@ NotationScene::layout(NotationStaff *singleStaff,
     setSceneRect(QRectF(0, 0, maxWidth, maxHeight));
 
     {
-        Profiler profiler("NotationScene::layout: regeneration", true);
+        //Profiler profiler("NotationScene::layout: regeneration", true);
 
     for (unsigned int i = 0; i < m_staffs.size(); ++i) {
 
@@ -1807,25 +1894,25 @@ NotationScene::handleEventRemoved(Event *e)
 }
 
 void
-NotationScene::setSelection(EventSelection *s,
+NotationScene::setSelection(EventSelection *selection,
                             bool preview)
 {
-    NOTATION_DEBUG << "NotationScene::setSelection: " << s;
+    //RG_DEBUG << "setSelection(): " << sselection;
 
-    if (!m_selection && !s) return;
-    if (m_selection == s) return;
-    if (m_selection && s && *m_selection == *s) {
+    if (!m_selection && !selection) return;
+    if (m_selection == selection) return;
+    if (m_selection && selection && *m_selection == *selection) {
         // selections are identical, no need to update elements, but
         // still need to replace the old selection to avoid a leak
         // (can't just delete s in case caller still refers to it)
         EventSelection *oldSelection = m_selection;
-        m_selection = s;
+        m_selection = selection;
         delete oldSelection;
         return;
     }
 
     EventSelection *oldSelection = m_selection;
-    m_selection = s;
+    m_selection = selection;
 
     NotationStaff *oldStaff = nullptr, *newStaff = nullptr;
 
@@ -1837,17 +1924,29 @@ NotationScene::setSelection(EventSelection *s,
         newStaff = setSelectionElementStatus(m_selection, true);
     }
 
-    if (newStaff) {
-        setCurrentStaff(newStaff);
+    timeT oldFrom = 0;
+    timeT oldTo = 0;
+    timeT newFrom = 0;
+    timeT newTo = 0;
+
+    if (oldSelection) {
+        oldFrom = std::min(oldSelection->getStartTime(),
+                           oldSelection->getNotationStartTime());
+        oldTo = std::max(oldSelection->getEndTime(),
+                         oldSelection->getNotationEndTime());
     }
+    if (m_selection) {
+        newFrom = std::min(m_selection->getStartTime(),
+                           m_selection->getNotationStartTime());
+        newTo = std::max(m_selection->getEndTime(),
+                         m_selection->getNotationEndTime());
+    }
+
+    RG_DEBUG << "From and to times:" << oldFrom << oldTo << newFrom << newTo;
 
     if (oldSelection && m_selection && oldStaff && newStaff &&
         (oldStaff == newStaff)) {
 
-        timeT oldFrom = oldSelection->getStartTime();
-        timeT oldTo = oldSelection->getEndTime();
-        timeT newFrom = m_selection->getStartTime();
-        timeT newTo = m_selection->getEndTime();
 
         // if the regions overlap, render once
         if ((oldFrom <= newFrom && oldTo >= newFrom) ||
@@ -1860,12 +1959,10 @@ NotationScene::setSelection(EventSelection *s,
         }
     } else {
         if (oldSelection && oldStaff) {
-        oldStaff->renderElements(oldSelection->getStartTime(),
-                                 oldSelection->getEndTime());
+            oldStaff->renderElements(oldFrom, oldTo);
         }
         if (m_selection && newStaff) {
-        newStaff->renderElements(m_selection->getStartTime(),
-                                 m_selection->getEndTime());
+            newStaff->renderElements(newFrom, newTo);
         }
     }
 
@@ -1873,7 +1970,7 @@ NotationScene::setSelection(EventSelection *s,
 
     delete oldSelection;
 
-    emit selectionChanged(m_selection);
+    emit selectionChangedES(m_selection);
     emit QGraphicsScene::selectionChanged();
 }
 
@@ -1889,12 +1986,12 @@ NotationScene::setSingleSelectedEvent(NotationStaff *staff,
 }
 
 void
-NotationScene::setSingleSelectedEvent(Segment *seg,
+NotationScene::setSingleSelectedEvent(Segment *segment,
                                       Event *e,
                                       bool preview)
 {
-    if (!seg || !e) return;
-    EventSelection *s = new EventSelection(*seg);
+    if (!segment || !e) return;
+    EventSelection *s = new EventSelection(*segment);
     s->addEvent(e);
     setSelection(s, preview);
 }
@@ -1906,6 +2003,7 @@ NotationScene::setSelectionElementStatus(EventSelection *s, bool set)
 
     NotationStaff *staff = nullptr;
 
+    // Find the NotationStaff for the EventSelection's Segment.
     for (std::vector<NotationStaff *>::iterator i = m_staffs.begin();
          i != m_staffs.end(); ++i) {
 
@@ -1917,17 +2015,21 @@ NotationScene::setSelectionElementStatus(EventSelection *s, bool set)
 
     if (!staff) return nullptr;
 
-    for (EventSelection::eventcontainer::iterator i = s->getSegmentEvents().begin();
+    for (EventContainer::iterator i = s->getSegmentEvents().begin();
          i != s->getSegmentEvents().end(); ++i) {
 
         Event *e = *i;
 
-        ViewElementList::iterator staffi = staff->findEvent(e);
-        if (staffi == staff->getViewElementList()->end()) continue;
+        ViewElementList::iterator staffElementIter = staff->findEvent(e);
+        // Not in the view?  Try the next.
+        if (staffElementIter == staff->getViewElementList()->end())
+            continue;
 
-        NotationElement *el = static_cast<NotationElement *>(*staffi);
+        NotationElement *element =
+                dynamic_cast<NotationElement *>(*staffElementIter);
 
-        el->setSelected(set);
+        if (element)
+            element->setSelected(set);
     }
 
     return staff;
@@ -1940,7 +2042,7 @@ NotationScene::previewSelection(EventSelection *s,
     if (!s) return;
     if (!m_document->isSoundEnabled()) return;
 
-    for (EventSelection::eventcontainer::iterator i = s->getSegmentEvents().begin();
+    for (EventContainer::iterator i = s->getSegmentEvents().begin();
          i != s->getSegmentEvents().end(); ++i) {
 
         Event *e = *i;
@@ -1973,15 +2075,18 @@ NotationScene::showPreviewNote(NotationStaff *staff, double layoutX,
     if (staff) {
         staff->showPreviewNote(layoutX, height, note, grace,
                                accidental, cautious, color);
+        m_previewNoteStaff = staff;
+
         if (play) playNote(staff->getSegment(), pitch, velocity);
     }
 }
 
 void
-NotationScene::clearPreviewNote(NotationStaff *staff)
+NotationScene::clearPreviewNote()
 {
-    if (staff) {
-        staff->clearPreviewNote();
+    if (m_previewNoteStaff) {
+        m_previewNoteStaff->clearPreviewNote();
+        m_previewNoteStaff = nullptr;
     }
 }
 
@@ -1998,6 +2103,7 @@ NotationScene::playNote(Segment &segment, int pitch, int velocity)
                                    RealTime(0, 250000000));
 }
 
+/* unused
 bool
 NotationScene::constrainToSegmentArea(QPointF &scenePos)
 {
@@ -2028,6 +2134,7 @@ NotationScene::constrainToSegmentArea(QPointF &scenePos)
 
     return ok;
 }
+*/
 
 bool
 NotationScene::isEventRedundant(Event *ev, Segment &seg) const
@@ -2081,6 +2188,7 @@ NotationScene::isEventRedundant(Key &key, timeT time, Segment &seg) const
     return key == previousKey;
 }
 
+/* unused
 bool
 NotationScene::isAnotherStaffNearTime(NotationStaff *currentStaff, timeT t)
 {
@@ -2106,6 +2214,7 @@ NotationScene::isAnotherStaffNearTime(NotationStaff *currentStaff, timeT t)
 
     return false;
 }
+*/
 
 void
 NotationScene::updateRefreshStatuses(TrackId track, timeT time)
@@ -2118,6 +2227,12 @@ NotationScene::updateRefreshStatuses(TrackId track, timeT time)
     }
 }
 
+void
+NotationScene::updatePageSize()
+{
+    layout(nullptr, 0, 0);
+}
+
 ///YG: Only for debug
 void
 NotationScene::dumpBarDataMap()
@@ -2127,5 +2242,3 @@ NotationScene::dumpBarDataMap()
 
 
 }
-
-
