@@ -3,7 +3,7 @@
 /*
     Rosegarden
     A MIDI and audio sequencer and musical notation editor.
-    Copyright 2000-2018 the Rosegarden development team.
+    Copyright 2000-2024 the Rosegarden development team.
 
     Other copyrights also apply to some parts of this work.  Please
     see the AUTHORS file and individual file headers for details.
@@ -97,9 +97,9 @@ TempoRuler::TempoRuler(RulerScale *rulerScale,
 
     m_editTempoController->setDocument(doc); // in case self() just created it
 
-    QObject::connect
-    (CommandHistory::getInstance(), SIGNAL(commandExecuted()),
-     this, SLOT(update()));
+    QObject::connect(
+            CommandHistory::getInstance(), &CommandHistory::commandExecuted,
+            this, static_cast<void(TempoRuler::*)()>(&TempoRuler::update));
 
     createAction("insert_tempo_here", SLOT(slotInsertTempoHere()));
     createAction("insert_tempo_at_pointer", SLOT(slotInsertTempoAtPointer()));
@@ -135,13 +135,15 @@ TempoRuler::mousePressEvent(QMouseEvent *e)
 
         if (e->type() == QEvent::MouseButtonDblClick) {
             timeT t = m_rulerScale->getTimeForX
-                      (e->x() - m_currentXOffset);
+                (e->pos().x() - m_currentXOffset);
             m_editTempoController->emitEditTempos(t);
             return;
         }
 
-        int x = e->x() + 1;
-        int y = e->y();
+        emit mousePress();
+
+        int x = e->pos().x() + 1;
+        int y = e->pos().y();
         timeT t = m_rulerScale->getTimeForX(x - m_currentXOffset);
         int tcn = m_composition->getTempoChangeNumberAt(t);
 
@@ -185,7 +187,7 @@ TempoRuler::mousePressEvent(QMouseEvent *e)
 
     } else if (e->button() == Qt::RightButton) {
 
-        m_clickX = e->x();
+        m_clickX = e->pos().x();
         if (!m_menu)
             createMenu();
         if (m_menu) {
@@ -202,13 +204,15 @@ TempoRuler::mousePressEvent(QMouseEvent *e)
 void
 TempoRuler::mouseReleaseEvent(QMouseEvent *e)
 {
+    emit mouseRelease();
+
     if (m_dragVert) {
 
         m_dragVert = false;
         unsetCursor();
 
-        if (e->x() < 0 || e->x() >= width() ||
-                e->y() < 0 || e->y() >= height()) {
+        if (e->pos().x() < 0 || e->pos().x() >= width() ||
+            e->pos().y() < 0 || e->pos().y() >= height()) {
             leaveEvent(nullptr);
         }
 
@@ -235,8 +239,8 @@ TempoRuler::mouseReleaseEvent(QMouseEvent *e)
         m_dragHoriz = false;
         unsetCursor();
 
-        if (e->x() < 0 || e->x() >= width() ||
-                e->y() < 0 || e->y() >= height()) {
+        if (e->pos().x() < 0 || e->pos().x() >= width() ||
+                e->pos().y() < 0 || e->pos().y() >= height()) {
             leaveEvent(nullptr);
         }
 
@@ -266,7 +270,7 @@ TempoRuler::mouseMoveEvent(QMouseEvent *e)
         if (shiftPressed != m_dragFine) {
 
             m_dragFine = shiftPressed;
-            m_dragStartY = e->y();
+            m_dragStartY = e->pos().y();
 
             // reset the start tempi to whatever we last updated them
             // to as we switch into or out of fine mode
@@ -277,7 +281,7 @@ TempoRuler::mouseMoveEvent(QMouseEvent *e)
             m_dragStartTarget = tr.first ? tr.second : -1;
         }
 
-        int diff = m_dragStartY - e->y(); // +ve for upwards drag
+        int diff = m_dragStartY - e->pos().y(); // +ve for upwards drag
         tempoT newTempo = m_dragStartTempo;
         tempoT newTarget = m_dragStartTarget;
 
@@ -318,7 +322,7 @@ TempoRuler::mouseMoveEvent(QMouseEvent *e)
 
     } else if (m_dragHoriz) {
 
-        int x = e->x();
+        int x = e->pos().x();
 
         SnapGrid grid(m_rulerScale);
         if (shiftPressed) {
@@ -351,7 +355,7 @@ TempoRuler::mouseMoveEvent(QMouseEvent *e)
 
     } else {
 
-        int x = e->x() + 1;
+        int x = e->pos().x() + 1;
         timeT t = m_rulerScale->getTimeForX(x - m_currentXOffset);
         int tcn = m_composition->getTempoChangeNumberAt(t);
 
@@ -402,7 +406,11 @@ TempoRuler::wheelEvent(QWheelEvent */* e */)
 {}
 
 void
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
+TempoRuler::enterEvent(QEnterEvent *)
+#else
 TempoRuler::enterEvent(QEvent *)
+#endif
 {
     TextFloat::getTextFloat()->attach(this);
     setMouseTracking(true);
@@ -442,7 +450,7 @@ TempoRuler::showTextFloat(tempoT tempo, tempoT target,
             m_composition->getMusicalTimeForAbsoluteTime
             (time, bar, beat, fraction, remainder);
             RealTime rt = m_composition->getElapsedRealTime(time);
-           
+
             // bars in user space start at 1, not 0
             bar++;
 
@@ -552,6 +560,7 @@ TempoRuler::getYForTempo(tempoT tempo)
     return y;
 }
 
+/* unused
 tempoT
 TempoRuler::getTempoForY(int y)
 {
@@ -569,6 +578,7 @@ TempoRuler::getTempoForY(int y)
 
     return tempo;
 }
+*/
 
 void
 TempoRuler::paintEvent(QPaintEvent* e)
@@ -586,7 +596,7 @@ TempoRuler::paintEvent(QPaintEvent* e)
     // using Thorn, use a nice dark gray that just contrasts with the black
     // horizontal line here
     QColor kuller(0x40, 0x40, 0x40);
-    if (!m_Thorn) kuller = palette().background().color();
+    if (!m_Thorn) kuller = palette().window().color();
     m_buffer.fill(kuller);
 
     QPainter paint(&m_buffer);
@@ -838,9 +848,9 @@ TempoRuler::paintEvent(QPaintEvent* e)
             prevBpm = bpm;
 
             QRect bounds = m_fontMetrics.boundingRect(tempoString);
-            
+
             x += 3; // bmp text aligns better in temporuler now - is this font dependent?
-            
+
             paint.setFont(m_font);
             if (time > 0)
                 x -= bounds.width() / 2;
