@@ -3,7 +3,7 @@
 /*
     Rosegarden
     A MIDI and audio sequencer and musical notation editor.
-    Copyright 2000-2018 the Rosegarden development team.
+    Copyright 2000-2024 the Rosegarden development team.
 
     Other copyrights also apply to some parts of this work.  Please
     see the AUTHORS file and individual file headers for details.
@@ -16,20 +16,90 @@
 */
 
 #define RG_MODULE_STRING "[ChannelManager]"
-//#define RG_NO_DEBUG_PRINT 1
+#define RG_NO_DEBUG_PRINT
 
 #include "ChannelManager.h"
 
 #include "base/AllocateChannels.h"
-#include "base/Instrument.h"
-#include "misc/Debug.h"
+#include "base/Composition.h"
 #include "misc/ConfigGroups.h"
+#include "misc/Debug.h"
+#include "base/Instrument.h"
 #include "sound/MappedEvent.h"
 #include "sound/MappedInserterBase.h"
 #include "sound/Midi.h"
+#include "misc/Preferences.h"
+#include "document/RosegardenDocument.h"
+#include "gui/application/RosegardenMainWindow.h"
 
 #include <QSettings>
 
+namespace
+{
+    bool allowReset()
+    {
+        static bool available = false;
+        static bool allowReset = true;
+
+        // Read only once when needed since QSettings is very slow.
+        // See bug #1589.
+
+        if (available)
+            return allowReset;
+
+        // Delayed read.  We can't do this in the ChannelManager ctor
+        // because there is a static instance in StudioControl and
+        // QSettings doesn't work right at static construction time.
+
+        QSettings settings;
+        settings.beginGroup(Rosegarden::SequencerOptionsConfigGroup);
+
+        // Allow CC 121 to be sent out to clear out CCs.
+        // Edit > Preferences... > MIDI > General > Allow Reset All Controllers
+        allowReset =
+                settings.value("allowresetallcontrollers", "true").toBool();
+        // Write back out so it is easy to find.
+        settings.setValue("allowresetallcontrollers", allowReset);
+
+        RG_DEBUG << "getSettings(): allowReset: " << allowReset;
+
+        available = true;
+
+        return allowReset;
+    }
+
+    bool forceChannelSetups()
+    {
+        static bool available = false;
+        static bool forceChannelSetups = false;
+
+        // Read only once when needed since QSettings is very slow.
+        // See bug #1589.
+
+        if (available)
+            return forceChannelSetups;
+
+        // Delayed read.  We can't do this in the ChannelManager ctor
+        // because there is a static instance in StudioControl and
+        // QSettings doesn't work right at static construction time.
+
+        QSettings settings;
+        settings.beginGroup(Rosegarden::SequencerOptionsConfigGroup);
+
+        // Related to Bug #1560
+        // When set to true, this causes MIPP channel setups to be sent at the
+        // beginning of each Segment for fixed channel Segments.
+        forceChannelSetups =
+                settings.value("forceChannelSetups", "false").toBool();
+        // Write back out so it is easy to find.
+        settings.setValue("forceChannelSetups", forceChannelSetups);
+
+        available = true;
+
+        return forceChannelSetups;
+    }
+}
+
 namespace Rosegarden
 {
 
@@ -100,61 +170,62 @@ void ChannelManager::insertChannelSetup(
         const Instrument *instrument,
         ChannelId channel,
         RealTime insertTime,
+        bool sendBSPC,
         const ControllerAndPBList &controllerAndPBList,
         MappedInserterBase &inserter)
 {
-    // Bank Select
+    if (sendBSPC) {
 
-    if (!instrument->hasFixedChannel()  ||
-        instrument->sendsBankSelect()) {
-        {
-            // Bank Select MSB
-            MappedEvent mE(instrument->getId(),
-                           MappedEvent::MidiController,
-                           MIDI_CONTROLLER_BANK_MSB,
-                           instrument->getMSB());
-            mE.setRecordedChannel(channel);
-            mE.setEventTime(insertTime);
-            mE.setTrackId(trackId);
-            inserter.insertCopy(mE);
+        // Bank Select
+
+        if (!instrument->hasFixedChannel()  ||
+            instrument->sendsBankSelect()) {
+            {
+                // Bank Select MSB
+                MappedEvent mE(instrument->getId(),
+                               MappedEvent::MidiController,
+                               MIDI_CONTROLLER_BANK_MSB,
+                               instrument->getMSB());
+                mE.setRecordedChannel(channel);
+                mE.setEventTime(insertTime);
+                mE.setTrackId(trackId);
+                inserter.insertCopy(mE);
+            }
+            {
+                // Bank Select LSB
+                MappedEvent mE(instrument->getId(),
+                               MappedEvent::MidiController,
+                               MIDI_CONTROLLER_BANK_LSB,
+                               instrument->getLSB());
+                mE.setRecordedChannel(channel);
+                mE.setEventTime(insertTime);
+                mE.setTrackId(trackId);
+                inserter.insertCopy(mE);
+            }
         }
-        {
-            // Bank Select LSB
-            MappedEvent mE(instrument->getId(),
-                           MappedEvent::MidiController,
-                           MIDI_CONTROLLER_BANK_LSB,
-                           instrument->getLSB());
-            mE.setRecordedChannel(channel);
-            mE.setEventTime(insertTime);
-            mE.setTrackId(trackId);
-            inserter.insertCopy(mE);
-        }
-    }
 
-    // Program Change
-
-    if (!instrument->hasFixedChannel()  ||
-        instrument->sendsProgramChange()) {
         // Program Change
-        MappedEvent mE(instrument->getId(),
-                       MappedEvent::MidiProgramChange,
-                       instrument->getProgramChange());
-        mE.setRecordedChannel(channel);
-        mE.setEventTime(insertTime);
-        mE.setTrackId(trackId);
-        inserter.insertCopy(mE);
+
+        if (!instrument->hasFixedChannel()  ||
+            instrument->sendsProgramChange()) {
+            // Program Change
+            MappedEvent mE(instrument->getId(),
+                           MappedEvent::MidiProgramChange,
+                           instrument->getProgramChange());
+            mE.setRecordedChannel(channel);
+            mE.setEventTime(insertTime);
+            mE.setTrackId(trackId);
+            inserter.insertCopy(mE);
+        }
+
     }
 
     // Reset All Controllers
 
-    // This is still desirable for some users.
-    QSettings settings;
-    settings.beginGroup(SequencerOptionsConfigGroup);
-    const bool allowReset =
-            settings.value("allowresetallcontrollers", "true").toBool();
-    settings.endGroup();
+    const StaticControllers &ccVector = controllerAndPBList.m_controllers;
 
-    if (allowReset) {
+    // If reset allowed and there are some CCs to send out
+    if (allowReset()  &&  !ccVector.empty()) {
         // In case some controllers are on that we don't know about, turn
         // all controllers off.  (Reset All Controllers)
         try {
@@ -173,11 +244,9 @@ void ChannelManager::insertChannelSetup(
 
     // Control Changes
 
-    const StaticControllers &list = controllerAndPBList.m_controllers;
-
     // For each controller
-    for (StaticControllerConstIterator cIt = list.begin();
-         cIt != list.end(); ++cIt) {
+    for (StaticControllers::const_iterator cIt = ccVector.begin();
+         cIt != ccVector.end(); ++cIt) {
         const MidiByte controlId = cIt->first;
         const MidiByte controlValue = cIt->second;
 
@@ -197,9 +266,9 @@ void ChannelManager::insertChannelSetup(
     // If there's a pitch bend, insert it...
     // We only do one type of pitchbend, though GM2 allows others.
     if (controllerAndPBList.m_havePitchbend) {
-        const int raised = controllerAndPBList.m_pitchbend + 8192;
-        const int d1 = (raised >> 7) & 0x7f;
-        const int d2 = raised & 0x7f;
+        const int pitchbend = controllerAndPBList.m_pitchbend;
+        const int d1 = (pitchbend >> 7) & 0x7f;
+        const int d2 = pitchbend & 0x7f;
 
         try {
             MappedEvent mE(instrument->getId(),
@@ -224,9 +293,7 @@ void ChannelManager::insertEvent(
         bool /*firstOutput*/,
         MappedInserterBase &inserter)
 {
-    //Profiler profiler("ChannelManager::insertEvent()", true);
-
-    //RG_DEBUG << "insertEvent(): playing on" << (m_instrument ? m_instrument->getPresentationName().c_str() : "nothing") << "at" << reftime << (firstOutput ? "needs init" : "doesn't need init");
+    //RG_DEBUG << "insertEvent(): playing on" << (m_instrument ? m_instrument->getPresentationName().c_str() : "nothing") << "at" << reftime;
 
     // ??? firstOutput was always ignored.  What would happen if we actually
     //     honored it?  E.g.:
@@ -272,24 +339,58 @@ bool ChannelManager::makeReady(
         // We already tried to get one and failed; don't keep trying.
         if (m_triedToGetChannel)
             return false;
-        
+
         // Try to get a channel.  This sets m_triedToGetChannel.
         allocateChannelInterval(false);
-        
+
         // If we still don't have one, give up.
         if (!m_channelInterval.validChannel())
             return false;
     }
-    
-    // If this instrument is in auto channels mode
-    if (!m_instrument->hasFixedChannel()) {
+
+    // Figure out whether playback is starting in the middle of a
+    // Segment.  Bug #1560.
+    // Note: This also causes channel setups on channel 10 due to the
+    //       MetronomeMapper.  Not sure if that's an issue, but might be.
+    bool startingInMiddle =
+            m_instrument->hasFixedChannel()  &&
+            (m_start < time  &&  time <= m_end);
+
+    // If this instrument is in auto channels mode or we are starting
+    // in the middle of a Segment.
+    if (!m_instrument->hasFixedChannel()  ||  forceChannelSetups()  ||
+        startingInMiddle) {
+
+        bool looping = false;
+        RosegardenMainWindow *mainWindow = RosegardenMainWindow::self();
+        if (mainWindow) {
+            // ??? What about LoopAll?
+            looping = (RosegardenDocument::currentDocument->
+                getComposition().getLoopMode() == Composition::LoopOn);
+        }
+
+        // This is for those who use looping as a compositional tool along
+        // with synths that can't handle program changes immediately prior
+        // to notes coming in.
+        const bool sendBSPC =
+                (Preferences::getSendProgramChangesWhenLooping()  ||
+                 !looping);
+
+        // This is for those who use looping as a compositional tool along
+        // with synths that can't handle control changes immediately prior
+        // to notes coming in.
+        const bool sendCCs =
+                (Preferences::getSendControlChangesWhenLooping()  ||
+                 !looping);
+
         insertChannelSetup(
                 trackId,
                 time,
-                controllerAndPBList,
+                sendBSPC,  // sendBSPC
+                sendCCs ? controllerAndPBList : ControllerAndPBList(),
                 inserter);
     }
-    
+
     m_ready = true;
 
     return true;
@@ -299,6 +400,7 @@ void
 ChannelManager::insertChannelSetup(
         TrackId trackId,
         RealTime insertTime,
+        bool sendBSPC,
         const ControllerAndPBList &controllerAndPBList,
         MappedInserterBase &inserter)
 {
@@ -315,7 +417,7 @@ ChannelManager::insertChannelSetup(
     if (m_instrument->getType() == Instrument::Midi) {
         ChannelId channel = m_channelInterval.getChannelId();
         insertChannelSetup(trackId, m_instrument, channel, insertTime,
-                           controllerAndPBList, inserter);
+                           sendBSPC, controllerAndPBList, inserter);
     }
 }
 
@@ -324,14 +426,14 @@ ChannelManager::setChannelIdDirectly()
 {
     Q_ASSERT(!m_usingAllocator);
 
-    ChannelId channel = m_instrument->getNaturalChannel();
+    ChannelId channel = m_instrument->getNaturalMidiChannel();
 
     if (m_instrument->getType() == Instrument::Midi) {
         // !!! Stopgap measure.  If we ever share allocators between
         // MIDI devices, this will have to become smarter.
         if (m_instrument->isPercussion()) {
             channel = (m_instrument->hasFixedChannel() ?
-                       m_instrument->getNaturalChannel() : 9);
+                       m_instrument->getNaturalMidiChannel() : 9);
         }
     }
 
@@ -398,7 +500,7 @@ ChannelManager::setAllocationMode(Instrument *instrument)
         if (m_usingAllocator != wasUsingAllocator)
             m_channelInterval.clearChannelId();
     }
-}    
+}
 
 void
 ChannelManager::allocateChannelInterval(bool changedInstrument)
@@ -496,7 +598,7 @@ ChannelManager::slotChannelBecomesFixed()
 {
     //RG_DEBUG << "slotChannelBecomesFixed()" << (m_usingAllocator ? "using allocator" : "not using allocator") << "for" << (void *)m_instrument;
 
-    ChannelId channel = m_instrument->getNaturalChannel();
+    ChannelId channel = m_instrument->getNaturalMidiChannel();
 
     // If we're already fixed and set to our natural channel, there's nothing
     // to do.
