@@ -3,7 +3,7 @@
 /*
     Rosegarden
     A MIDI and audio sequencer and musical notation editor.
-    Copyright 2000-2018 the Rosegarden development team.
+    Copyright 2000-2024 the Rosegarden development team.
 
     Other copyrights also apply to some parts of this work.  Please
     see the AUTHORS file and individual file headers for details.
@@ -25,7 +25,6 @@
 #include "base/Exception.h"
 #include "base/MidiTypes.h"
 #include "base/NotationTypes.h"
-#include "base/Profiler.h"
 #include "base/RealTime.h"
 #include "base/Segment.h"
 #include "base/SegmentPerformanceHelper.h"
@@ -61,7 +60,7 @@ RealTime
 InternalSegmentMapper::
 toRealTime(Composition &comp, timeT t)
 {
-    return 
+    return
         comp.getElapsedRealTime(t) + m_segment->getRealTimeDelay();
 }
 
@@ -90,18 +89,17 @@ void InternalSegmentMapper::fillBuffer()
     resize(0);
 
 #ifdef DEBUG_INTERNAL_SEGMENT_MAPPER
-    SEQMAN_DEBUG
-        << "InternalSegmentMapper::fillBuffer"
+    RG_DEBUG
+        << "fillBuffer(): "
         << (void *)this
         << "Segment"
         << (void *)m_segment
         << "with repeat count"
-        << repeatCount
-        << endl;
+        << repeatCount;
 #endif
-    
+
     // Clear out stuff from before.
-    m_triggeredEvents->clear(); 
+    m_triggeredEvents->clear();
     m_controllerCache.clear();
     m_noteOffs = NoteoffContainer();
 
@@ -116,7 +114,7 @@ void InternalSegmentMapper::fillBuffer()
         // on.  Eg, on the second time thru we play everything one
         // segment duration later and so forth.
         timeT timeForRepeats = repeatNo * segmentDuration;
-        
+
         for (Segment::iterator j = m_segment->begin();
              m_segment->isBeforeEndMarker(j) ||
                  (implied != m_triggeredEvents->end());
@@ -132,7 +130,7 @@ void InternalSegmentMapper::fillBuffer()
             if (m_segment->isBeforeEndMarker(j)) {
                 bestBaseTime = (*j)->getAbsoluteTime();
             }
-            
+
             // k is a pointer to the note iterator we will actually
             // use.  Initialize it to the default of the segment's
             // own.
@@ -158,7 +156,7 @@ void InternalSegmentMapper::fillBuffer()
 
             // We handle nested ornament expansion elsewhere, so
             // trigger events won't be found in implied.
-            if (!usingImplied) { 
+            if (!usingImplied) {
 
                 long triggerId = -1;
                 (**k)->get<Int>(BaseProperties::TRIGGER_SEGMENT_ID, triggerId);
@@ -198,9 +196,9 @@ void InternalSegmentMapper::fillBuffer()
                             reserve(spaceNeeded);
                         }
                     }
-                        
+
                     // whatever happens, we don't want to write this one
-                    ++j; 
+                    ++j;
 
                     // Since we're no longer sure what the next event
                     // is, restart the loop.
@@ -255,7 +253,7 @@ void InternalSegmentMapper::fillBuffer()
                                 (**k)->isa(PitchBend::EventType)) {
                                 m_controllerCache.storeLatestValue((**k));
                             }
-                            
+
                             if ((**k)->isa(Note::EventType)) {
                                 if (m_segment->getTranspose() != 0) {
                                     e.setPitch(e.getPitch() +
@@ -266,12 +264,12 @@ void InternalSegmentMapper::fillBuffer()
                                                    e.getPitch());
                                 }
                             }
-                            mapAnEvent(&e); 
+                            mapAnEvent(&e);
                         } else {}
-                        
+
                     } catch (...) {
 #ifdef DEBUG_INTERNAL_SEGMENT_MAPPER
-                        SEQMAN_DEBUG << "SegmentMapper::fillBuffer - caught exception while trying to create MappedEvent\n";
+                        RG_DEBUG << "fillBuffer() - caught exception while trying to create MappedEvent";
 #endif
                     }
                 }
@@ -297,16 +295,16 @@ void InternalSegmentMapper::fillBuffer()
 
         // Fix for bug #1378.  Start slightly before the first note so
         // that program etc is sent then.  We'll allow it to be before
-        // zeroTime, since MappedBufMetaIterator can handle early
+        // zeroTimeF(), since MappedBufMetaIterator can handle early
         // start-times.
         static const RealTime preparationTime = RealTime::fromSeconds(0.5);
         minRealTime = minRealTime - preparationTime;
     } else {
-        minRealTime = maxRealTime = RealTime::zeroTime;
+        minRealTime = maxRealTime = RealTime::zero();
     }
 
     m_channelManager.setRequiredInterval(minRealTime, maxRealTime,
-                                         RealTime::zeroTime, RealTime(1,0));
+                                         RealTime::zero(), RealTime(1,0));
 
     // If the track is making sound
     if (!ControlBlock::getInstance()->isTrackMuted(track->getId())  &&
@@ -349,8 +347,7 @@ enqueueNoteoff(timeT time, int pitch)
          i != m_noteOffs.end(); ++i) {
         if (i->second == pitch) {
 #ifdef DEBUG_INTERNAL_SEGMENT_MAPPER
-            SEQMAN_DEBUG << "duplicated NOTE OFF  pitch: " << pitch
-                         << " at " << time << std::endl;
+            RG_DEBUG << "enqueueNoteoff(): duplicated NOTE OFF  pitch: " << pitch << " at " << time;
 #endif
             break;
         }
@@ -382,7 +379,7 @@ popInsertNoteoff(int trackid, Composition &comp)
 }
 
 int
-InternalSegmentMapper::addSize(int size, Segment *s)
+InternalSegmentMapper::addSize(int size, Segment *s) const
 {
     int repeatCount = getSegmentRepeatCount();
     // Double the size because we may get a noteoff for every noteon
@@ -396,13 +393,17 @@ InternalSegmentMapper::calculateSize()
     return addSize(0, m_segment);
 }
 
-// Make the channel ready to be played on.  
+// Make the channel ready to be played on.
 void
 InternalSegmentMapper::
 makeReady(MappedInserterBase &inserter, RealTime time)
 {
     Instrument *instrument = m_doc->getInstrument(m_segment);
+    if (!instrument)
+        return;
+
     m_channelManager.setInstrument(instrument);
+
     m_channelManager.makeReady(
             m_segment->getTrack(),
             time,
@@ -414,6 +415,8 @@ void
 InternalSegmentMapper::insertChannelSetup(MappedInserterBase &inserter)
 {
     Instrument *instrument = m_doc->getInstrument(m_segment);
+    if (!instrument)
+        return;
 
     // If this segment's Instrument is in Auto channels mode, bail.
     if (!instrument->hasFixedChannel())
@@ -422,18 +425,21 @@ InternalSegmentMapper::insertChannelSetup(MappedInserterBase &inserter)
     m_channelManager.setInstrument(instrument);
     m_channelManager.insertChannelSetup(
             m_segment->getTrack(),
-            RealTime::zeroTime,
-            getControllers(instrument, RealTime::zeroTime),
+            RealTime::zero(),
+            true,  // sendBSPC
+            getControllers(instrument, RealTime::zero()),
             inserter);
 }
 
 void
 InternalSegmentMapper::doInsert(MappedInserterBase &inserter, MappedEvent &evt,
-                               RealTime start, bool dirtyIter)
+                               RealTime start, bool firstOutput)
 {
     Instrument *instrument = m_doc->getInstrument(m_segment);
+    if (!instrument)
+        return;
 
-    if (dirtyIter)
+    if (firstOutput)
         m_channelManager.setInstrument(instrument);
 
     m_channelManager.insertEvent(
@@ -441,13 +447,13 @@ InternalSegmentMapper::doInsert(MappedInserterBase &inserter, MappedEvent &evt,
             getControllers(instrument, start),
             start,
             evt,
-            dirtyIter,  // firstOutput
+            firstOutput,
             inserter);
 }
 
 int
 InternalSegmentMapper::
-getControllerValue(timeT searchTime, const std::string eventType,
+getControllerValue(timeT searchTime, const std::string& eventType,
                    int controllerId)
 {
     return
@@ -458,7 +464,7 @@ getControllerValue(timeT searchTime, const std::string eventType,
 
 bool
 InternalSegmentMapper::
-shouldPlay(MappedEvent *evt, RealTime sliceStart)
+shouldPlay(MappedEvent *evt, RealTime startTime)
 {
     // #1048388:
     // Ensure sysex heeds mute status, but ensure clocks etc still get
@@ -466,55 +472,68 @@ shouldPlay(MappedEvent *evt, RealTime sliceStart)
     if (evt->getType() == MappedEvent::MidiSystemMessage &&
         evt->getData1() != MIDI_SYSTEM_EXCLUSIVE)
         { return true; }
-    
+
     // Otherwise if it's muted it doesn't play.
     if (mutedEtc()) { return false; }
 
     // Otherwise it should play if it's not already all done sounding.
     // The timeslice logic will have already excluded events that
     // start too late.
-    return !evt->EndedBefore(sliceStart);
+    return !evt->EndedBefore(startTime);
 }
 
 ControllerAndPBList
 InternalSegmentMapper::getControllers(Instrument *instrument, RealTime start)
 {
-    Profiler profiler("InternalSegmentMapper::getControllers()", false);
+    if (!instrument)
+        return ControllerAndPBList();
+
+#ifdef DEBUG_INTERNAL_SEGMENT_MAPPER
+    RG_DEBUG << "getControllers()...";
+
+    StaticControllers &controllers2 = instrument->getStaticControllers();
+    // for each controller...
+    for (const ControllerValuePair &controller : controllers2) {
+        RG_DEBUG << "  controller/value:" << controller.first << "/" << controller.second;
+    }
+#endif
+
     timeT startTime =
         m_doc->getComposition().getElapsedTimeForRealTime(start);
 
     // If time is at or before all events, we can just use static values.
-    if (startTime <= m_segment->getStartTime()) {
+    if (startTime <= m_segment->getStartTime())
         return ControllerAndPBList(instrument->getStaticControllers());
-    }
 
     ControllerAndPBList returnValue;
-    // Get a value for each controller type.  Always get a value, just
-    // in case the controller used to be set on this channel.
-    StaticControllers &list = instrument->getStaticControllers();
-    for (StaticControllerConstIterator cIt = list.begin();
-         cIt != list.end(); ++cIt) {
-        MidiByte controlId = cIt->first;
+
+    const StaticControllers &controllers = instrument->getStaticControllers();
+
+    // For each controller
+    for (const ControllerValuePair &controller : controllers) {
+        // Always get a value for every controller type, just in case the
+        // controller used to be set on this channel.
+
+        MidiByte controllerId = controller.first;
+
         MidiByte controlValue =
-            getControllerValue(startTime,
-                               Controller::EventType,
-                               controlId);
-        returnValue.m_controllers.
-            push_back(std::pair<MidiByte, MidiByte>(controlId,
-                                                    controlValue));
+                getControllerValue(startTime,
+                                   Controller::EventType,
+                                   controllerId);
+
+        returnValue.m_controllers.push_back(
+                ControllerValuePair(controllerId, controlValue));
     }
 
     // Do the same for PitchBend, of which we only treat one (ie, we
     // ignore GM2's other PitchBend types)
-    {
-        MidiByte controlValue =
-            getControllerValue(startTime,
-                               PitchBend::EventType,
-                               0);
 
-        returnValue.m_havePitchbend = true;
-        returnValue.m_pitchbend     = controlValue;
-    }
+    returnValue.m_havePitchbend = true;
+    returnValue.m_pitchbend =
+            getControllerValue(startTime,  // searchTime
+                               PitchBend::EventType,  // eventType
+                               0);  // controllerId
+
     return returnValue;
 }
 
