@@ -3,7 +3,7 @@
 /*
     Rosegarden
     A MIDI and audio sequencer and musical notation editor.
-    Copyright 2000-2018 the Rosegarden development team.
+    Copyright 2000-2024 the Rosegarden development team.
 
     Other copyrights also apply to some parts of this work.  Please
     see the AUTHORS file and individual file headers for details.
@@ -16,6 +16,7 @@
 */
 
 #define RG_MODULE_STRING "[MidiInserter]"
+#define RG_NO_DEBUG_PRINT 1
 
 #include "MidiInserter.h"
 
@@ -53,8 +54,7 @@ insertMidiEvent(MidiEvent *event)
     event->setTime(delta);
 #ifdef MIDI_DEBUG
     RG_DEBUG << "Converting absoluteTime" << (int)absoluteTime
-             << "to delta" << (int)delta
-             << endl;
+             << "to delta" << (int)delta;
 #endif
     m_midiTrack.push_back(event);
 }
@@ -100,7 +100,6 @@ MidiInserter(Composition &composition, int timingDivision, RealTime trueEnd) :
     m_timingDivision(timingDivision),
     m_finished(false),
     m_trueEnd(trueEnd),
-    m_previousRealTime(RealTime::zeroTime),
     m_previousTime(0),
     m_ramping(false)
 { setup(); }
@@ -111,15 +110,15 @@ MidiInserter(Composition &composition, int timingDivision, RealTime trueEnd) :
 // @author Tom Breton (Tehom)
 timeT
 MidiInserter::
-getAbsoluteTime(RealTime realtime)
+getAbsoluteTime(RealTime realtime) const
 {
     timeT time = m_comp.getElapsedTimeForRealTime(realtime);
+    RG_DEBUG << "getAbsoluteTime" << realtime << time;
     timeT retVal = (time * m_timingDivision) / crotchetDuration;
 #ifdef MIDI_DEBUG
     RG_DEBUG << "Converting RealTime" << realtime
              << "to timeT" << retVal
-             << "intermediate" << time
-             << endl;
+             << "intermediate" << time;
 #endif
 
     return retVal;
@@ -130,7 +129,7 @@ getAbsoluteTime(RealTime realtime)
 // Adapted from MidiFile.cpp
 void
 MidiInserter::
-initNormalTrack(TrackData &trackData, TrackId RGTrackPos)
+initNormalTrack(TrackData &trackData, TrackId RGTrackPos) const
 {
     Track *track = m_comp.getTrackById(RGTrackPos);
     trackData.m_previousTime = 0;
@@ -153,7 +152,7 @@ getTrackData(TrackId RGTrackPos, int channelNb)
 #endif
     // Some events like TimeSig and Tempo have invalid trackId and
     // should be written on the conductor track.
-    if (RGTrackPos == NO_TRACK)
+    if (RGTrackPos == NoTrack)
         { return m_conductorTrack; }
     // Otherwise we're looking it up.
     TrackKey key = TrackKey(RGTrackPos, channelNb);
@@ -173,7 +172,7 @@ MidiInserter::
 setup()
 {
     m_conductorTrack.m_previousTime = 0;
-    
+
     // Insert the Rosegarden Signature Track here and any relevant
     // file META information - this will get written out just like
     // any other MIDI track.
@@ -224,6 +223,13 @@ insertCopy(const MappedEvent &evt)
     TrackData& trackData   = getTrackData(evt.getTrackId(), midiChannel);
     timeT      midiEventAbsoluteTime = getAbsoluteTime(evt.getEventTime());
 
+#ifdef BUG1627
+    // to avoid negative times here we subtract the start time
+    timeT start = m_comp.getStartMarker();
+    start = (start * m_timingDivision) / crotchetDuration;
+    midiEventAbsoluteTime -= start;
+#endif
+
     // If we are ramping, calculate a previous tempo that would get us
     // to this event at this time and pre-insert it, unless this
     // event's time is the same as last.
@@ -255,6 +261,8 @@ insertCopy(const MappedEvent &evt)
                     // Yes, we fetch it from "instrument" because
                     // that's what TempoSegmentMapper puts it in.
                     tempoT tempo = evt.getInstrument();
+                    RG_DEBUG << "insertCopy tempo" << evt.getEventTime() <<
+                        midiEventAbsoluteTime << tempo;
                     trackData.insertTempo(midiEventAbsoluteTime, tempo);
                     break;
                 }
@@ -279,7 +287,7 @@ insertCopy(const MappedEvent &evt)
                     }
 
                     timeSigString += (MidiByte) denPowerOf2;
-                
+
                     // The third byte is the number of MIDI clocks per beat.
                     // There are 24 clocks per quarter-note (the MIDI clock
                     // is tempo-independent and is not related to the timebase).
@@ -329,6 +337,8 @@ insertCopy(const MappedEvent &evt)
                     MidiByte pitch         = evt.getData1();
                     MidiByte midiVelocity  = evt.getData2();
 
+                    RG_DEBUG << "insertCopy note" << evt.getEventTime() <<
+                        midiEventAbsoluteTime << pitch << midiVelocity;
                     if ((evt.getType() == MappedEvent::MidiNote) &&
                         (midiVelocity == 0)) {
                         // It's actually a NOTE_OFF.
@@ -365,7 +375,7 @@ insertCopy(const MappedEvent &evt)
 
             case MappedEvent::MidiSystemMessage:
                 {
-                    std::string data = 
+                    std::string data =
                         DataBlockRepository::getInstance()->
                         getDataBlockForEvent(&evt);
 
@@ -409,7 +419,7 @@ insertCopy(const MappedEvent &evt)
 
             case MappedEvent::Marker:
                 {
-                    std::string metaMessage = 
+                    std::string metaMessage =
                         DataBlockRepository::getInstance()->
                         getDataBlockForEvent(&evt);
 
@@ -427,7 +437,7 @@ insertCopy(const MappedEvent &evt)
                 {
                     MidiByte midiTextType = evt.getData1();
 
-                    std::string metaMessage = 
+                    std::string metaMessage =
                         DataBlockRepository::getInstance()->
                         getDataBlockForEvent(&evt);
 
@@ -440,7 +450,18 @@ insertCopy(const MappedEvent &evt)
                     break;
                 }
 
-            
+            case MappedEvent::KeySignature:
+                {
+                    std::string metaMessage;
+                    metaMessage += MidiByte(evt.getData1());
+                    metaMessage += MidiByte(evt.getData2());
+
+                    trackData.insertMidiEvent(
+                        new MidiEvent(midiEventAbsoluteTime,
+                                      MIDI_FILE_META_EVENT,
+                                      MIDI_KEY_SIGNATURE,
+                                      metaMessage));
+                }
                 // Pacify compiler warnings about missed cases.
             case MappedEvent::InvalidMappedEvent:
             case MappedEvent::Audio:
@@ -463,22 +484,18 @@ insertCopy(const MappedEvent &evt)
             default:
                 break;
             }
-    } catch (MIDIValueOutOfRange r) {
-#ifdef MIDI_DEBUG
-        RG_DEBUG << "MIDI value out of range at " << midiEventAbsoluteTime;
-#endif
 
-    } catch (Event::NoData d) {
+    } catch (const Event::NoData &d) {
 #ifdef MIDI_DEBUG
         RG_DEBUG << "Caught Event::NoData at " << midiEventAbsoluteTime << ", message is:" << d.getMessage();
 #endif
 
-    } catch (Event::BadType b) {
+    } catch (const Event::BadType &b) {
 #ifdef MIDI_DEBUG
         RG_DEBUG << "Caught Event::BadType at " << midiEventAbsoluteTime << ", message is:" << b.getMessage();
 #endif
 
-    } catch (SystemExclusive::BadEncoding e) {
+    } catch (const SystemExclusive::BadEncoding &e) {
 #ifdef MIDI_DEBUG
         RG_DEBUG << "Caught bad SysEx encoding at " << midiEventAbsoluteTime;
 #endif
